import * as https from 'https';
import * as querystring from 'node:querystring';


export class DirectPost {
    security_key: string;
    billing: any;
    shipping: any;

    constructor(security_key: string) {
        this.security_key = security_key;
    }

    setBilling(billingInformation: any) {
        // Validate that passed in information contains valid keys
        const validBillingKeys = ['first_name', 'last_name', 'company', 'address1',
            'address2', 'city', 'state', 'zip', 'country', 'phone', 'fax', 'email'];

        for (let key in billingInformation) {
            if (!validBillingKeys.includes(key)) {
                throw new Error(`Invalid key provided in billingInformation. '${key}'
            is not a valid billing parameter.`)
            }
        }
        ;

        this.billing = billingInformation;
    }

    setShipping(shippingInformation: any) {
        // Validate that passed in information contains valid keys
        const validShippingKeys = [
            'shipping_first_name', 'shipping_last_name', 'shipping_company',
            'shipping_address1', 'address2', 'shipping_city', 'shipping_state',
            'shipping_zip', 'shipping_country', 'shipping_email'
        ];

        for (let key in shippingInformation) {
            if (!validShippingKeys.includes(key)) {
                throw new Error(`Invalid key provided in shippingInformation. '${key}'
            is not a valid shipping parameter.`)
            }
        }
        ;

        this.shipping = shippingInformation;
    }

    async doSale(amount: any, ccNum: any, ccExp: any, cvv: any) {
        let requestOptions = {
            'type': 'sale',
            'amount': amount,
            'ccnumber': ccNum,
            'ccexp': ccExp,
            'cvv': cvv
        };

        // Merge together all request options into one object
        Object.assign(requestOptions, this.billing, this.shipping);

        // Make request
        return await this._doRequest(requestOptions);
    }

    async doRefund(amount: any, transactionId: any) {
        let requestOptions = {
            'type': 'refund',
            'amount': amount,
            'transactionid': transactionId
        };

        // Make request
        return await this._doRequest(requestOptions);
    }

    async _doRequest(postData: any) {
        const hostName = 'secure.nmi.com';
        const path = '/api/transact.php';

        postData.security_key = this.security_key;
        postData = querystring.stringify(postData);

        const options = {
            hostname: hostName,
            path: path,
            method: 'POST',
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Content-Length': Buffer.byteLength(postData)
            }
        };

        return new Promise((resolve, reject) => {
            // Make request to Payment API
            const req = https.request(options, (response) => {
                let data = [];
                response.on('data', (chunk) => {
                    data.push(chunk);
                });
                response.on('end', () => {
                    let resData: any;
                    const body = Buffer.concat(data);
                    const responseData = body.toString()
                    resData = querystring.parse(responseData);
                    resData['statusCode'] = response.statusCode;
                    resolve(resData);
                });
            });

            req.on('error', (e) => {
                reject(e);
                console.error(`Problem with request: ${e.message}`);
            });

            // Write post data to request body
            req.write(postData);
            req.end();
        });
    }
}

