import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from '@nestjs/common';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {Constants} from 'src/utils/constants';
import {FooterHelpSectionService} from './footer-help-section.service';
import {CreateFooterHelpSectionDto} from './dto/create-footer-help-section.dto';
import {UpdateFooterHelpSectionDto} from './dto/update-footer-help-section.dto';

@Controller({path: 'footer-help-section', version: Constants.API_VERSION_1})
export class FooterHelpSectionController {
    constructor(private readonly service: FooterHelpSectionService) {
    }

    @Post()
    // @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateFooterHelpSectionDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateFooterHelpSectionDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
