import {EntityRepository, getConnection, Repository} from "typeorm";
import {ValuesSectionEntity} from "./entities/values-section.entity";
import {CreateValuesSectionDto} from "./dto/create-values-section.dto";
import {UpdateValuesSectionDto} from "./dto/update-values-section.dto";

@EntityRepository(ValuesSectionEntity)
export class ValuesSectionRepository extends Repository<ValuesSectionEntity> {
    async createEntity(dto: CreateValuesSectionDto): Promise<ValuesSectionEntity | Error> {
        try {
            const {
                image,
                title,
                subtitle
            } = dto;
            const entity = this.create({
                image,
                title,
                subtitle
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ValuesSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ValuesSectionEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ValuesSectionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateValuesSectionDto): Promise<ValuesSectionEntity | Error> {
        try {
            await this.update(id, {
                image: dto.image,
                title: dto.title,
                subtitle: dto.subtitle,
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }
}