// doSpacesService.ts
import {Inject, Injectable} from '@nestjs/common';
import * as AWS from 'aws-sdk';
import {extname} from 'path';
import {DoSpacesServiceLib, UploadedMulterFileI} from '.';
import {v4 as uuidv4} from 'uuid';

// Typical nestJs service
@Injectable()
export class DoSpacesService {
    constructor(@Inject(DoSpacesServiceLib) private readonly s3: AWS.S3) {
    }

    async uploadFile(file: UploadedMulterFileI, uploadLocation?: string) {
        // Precaution to avoid having 2 files with the same name
        const extension = extname(file.originalname);
        let fileName = `hymanfashion.${uuidv4()}_do${extension}`;
        if (uploadLocation) {
            fileName = uploadLocation + fileName;
        }

        // Return a promise that resolves only when the file upload is complete
        return new Promise((resolve, reject) => {
            this.s3.putObject(
                {
                    Bucket: 'hyman-fashion',
                    Key: fileName,
                    Body: file.buffer,
                    ACL: 'public-read',
                    ContentType: file.mimetype
                },
                (error: AWS.AWSError) => {
                    if (!error) {
                        resolve(`https://hyman-fashion.nyc3.cdn.digitaloceanspaces.com/${fileName}`);
                    } else {
                        reject(
                            new Error(
                                `DoSpacesService_ERROR: ${error.message || 'Something went wrong'}`,
                            ),
                        );
                    }
                },
            );
        });
    }

    async uploadFileBuffer(file: any, extension: any, uploadLocation?: string) {
        // Precaution to avoid having 2 files with the same name
        let fileName = `hymanfashion.${uuidv4()}_do.${extension}`;
        if (uploadLocation) {
            fileName = uploadLocation + fileName;
        }

        // Return a promise that resolves only when the file upload is complete
        return new Promise((resolve, reject) => {
            this.s3.putObject(
                {
                    Bucket: 'hyman-fashion',
                    Key: fileName,
                    Body: file,
                    ACL: 'public-read',
                },
                (error: AWS.AWSError) => {
                    if (!error) {
                        resolve(`https://hyman-fashion.nyc3.cdn.digitaloceanspaces.com/${fileName}`);
                    } else {
                        reject(
                            new Error(
                                `DoSpacesService_ERROR: ${error.message || 'Something went wrong'}`,
                            ),
                        );
                    }
                },
            );
        });
    }
}