import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreatePaymentMethodDto} from "./dto/create-payment-method.dto";
import {UpdatePaymentMethodDto} from "./dto/update-payment-method.dto";
import {PaymentMethod} from "./entity/payment-method.entity";

@EntityRepository(PaymentMethod)
export class PaymentMethodRepository extends Repository<PaymentMethod> {
    async createEntity(
        dto: CreatePaymentMethodDto
    ): Promise<PaymentMethod | Error> {
        try {
            const {name} = dto;
            const entity = this.create({
                name
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(
        id: string
    ): Promise<PaymentMethod | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(
        id: string,
        dto: UpdatePaymentMethodDto
    ): Promise<PaymentMethod | Error> {
        try {
            await this.update(id, {
                name: dto.name
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<PaymentMethod[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(PaymentMethod)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async findByName(name: any): Promise<PaymentMethod[] | Error> {
        try {
            return await this.find({
                where: {name: name, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }
}