import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CreateProductDumpImportDto } from './dto/create-product-dump-import.dto';
import { UpdateProductDumpImportDto } from './dto/update-product-dump-import.dto';
import { ProductDumpImport } from './entity/product-dump-import.entity';
import { ProductDumpImportRepository } from './product-dump-importrepository';

@Injectable()
export class ProductDumpImportService {

    constructor(
        @InjectRepository(ProductDumpImportRepository)
        private readonly repository: ProductDumpImportRepository,
        private readonly fileService: FileUploadsService,
    ) { }

    async create(dto: CreateProductDumpImportDto): Promise<ProductDumpImport | Error> {

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data created","data", data);
    }

    async findAll(): Promise<ProductDumpImport[] | Error> {
      
        const data = await this.repository.findByFilterQuery({});
        
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data fetched","data", data);
    }

    async update(id: string, dto: UpdateProductDumpImportDto): Promise<ProductDumpImport | Error> {

        const dataPrevious : any = await this.repository.findOneEntity(id);
        if(!dataPrevious){
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data updated","data", data);
    }

    async findOne(id: string): Promise<ProductDumpImport[] | Error> {
      
        const data = await this.repository.findByFilterQuery({id: id});
        
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data fetched","data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.buildDeletedData(
                data,
        );
    }


}
