import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from '@nestjs/common';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { Constants } from 'src/utils/constants';
import { CreateProductSaleVariantsDto } from './dto/create-product-sale-variants.dto';
import { UpdateProductSaleVariantsDto } from './dto/update-product-sale-variants.dto';
import { ProductSaleVariantsService } from './product-sale-variants.service';

@Controller({ path: 'product-sale-variants', version: Constants.API_VERSION_1 })
export class ProductSaleVariantsController {
    constructor(private readonly service: ProductSaleVariantsService) {}

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateProductSaleVariantsDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateProductSaleVariantsDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
