import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductSaleRepository } from './product-sale.repository';
import { CreateProductSaleDto } from './dto/create-product-sale.dto';
import { ProductSale } from './entities/product-sale.entity';
import { UpdateProductSaleDto } from './dto/update-product-sale.dto';
import { urlSlug } from 'src/helper/common-helper';

@Injectable()
export class ProductSaleService {
    constructor(
        @InjectRepository(ProductSaleRepository)
        private readonly repository: ProductSaleRepository
    ) { }

    async create(dto: CreateProductSaleDto): Promise<ProductSale | Error> {
        const res: any = await this.repository.findByName(dto.name);
        if (res.length > 0) {
            throw new HttpException(
                "Product sale name already exists.",
                HttpStatus.BAD_REQUEST
            );
        }

        let slug = urlSlug(dto.name);
        dto.urlSlug = slug;

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product sale created successfully", "data", data);
    }

    async findAll(): Promise<ProductSale[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product sale fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductSaleDto): Promise<ProductSale | Error> {
        const category: any = await this.repository.findOneEntity(id);
        if (!category) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        let slug = urlSlug(dto.name);
        dto.urlSlug = slug;

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product sale updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductSale[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product sale fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product sale deleted successfully");
    }
}
