import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { AboutUsPageCmsRepository } from '../CMS/about-us-page-cms/about-us-page-cms.repository';
import { AccountDashboardCmsRepository } from '../CMS/account-dashboard-cms/account-dashboard-cms.repository';
import { BlogPageCmsRepository } from '../CMS/blog-page-cms/blog-page-cms.repository';
import { ClearenceSaleSectionCmsRepository } from '../CMS/clearence-sale-section-cms/clearence-sale-section-cms.repository';
import { ContactUsPageCmsRepository } from '../CMS/contact-us-page-cms/contact-us-page-cms.repository';
import { FaqPageCmsRepository } from '../CMS/faq-page-cms/faq-page-cms.repository';
import { FooterCmsRepository } from '../CMS/footer-cms/footer-cms.repository';
import { HeaderBannerCmsRepository } from '../CMS/header-banner-cms/header-banner-cms.repository';
import { InstagramSectionCmsRepository } from '../CMS/instagram-section-cms/instagram-section-cms.repository';
import { LatestArrivalCmsRepository } from '../CMS/latest-arrival-cms/latest-arrival-cms.repository';
import { LatestNewsCmsRepository } from '../CMS/latest-news-cms/latest-news-cms.repository';
import { LooksYouWillLoveCmsRepository } from '../CMS/looks-you-will-love-cms/looks-you-will-love-cms.repository';
import { PrivacyPageCmsRepository } from '../CMS/privacy-page-cms/privacy-cms.repository';
import { RefundPageCmsRepository } from '../CMS/refund-page-cms/refund-cms.repository';
import { ShopPageCmsRepository } from '../CMS/shop-page-cms/shop-page-cms.repository';
import { TrendingSectionCmsRepository } from '../CMS/trending-section-cms/trending-section-cms.repository';
import { FAQRepository } from '../CRUD/faq/faq.repository';
import { FooterCopyrightSectionRepository } from '../CRUD/footer-copyright-section/footer-copyright-section.repository';
import { InstagramServiceRepository } from '../CRUD/instagram-service/instagram-service.repository';
import { LatestCollectionSectionRepository } from '../CRUD/latest-collection-section/latest-collection-section.repository';
import { SocialMediaLinkRepository } from '../CRUD/social-media-link/social-media-link.repository';
import { ValuesSectionRepository } from '../CRUD/values-section/values-section.repository';
import { BillingAddressRepository } from '../address/billing-address/billing-address.repository';
import { ShippingAddressRepository } from '../address/shipping-address/shipping-address.repository';
import { BlogPostRepository } from '../blog/blog-post/blog-post.repository';
import { BlogPost } from '../blog/blog-post/entities/blog-post.entity';
import { ProductCareVariantsRepository } from '../product/product-care-variants/product-care-variants.repository';
import { ProductCategory } from '../product/product-category/entities/product-category.entity';
import { ProductCategoryRepository } from '../product/product-category/product-category.repository';
import { ProductColorVariantsRepository } from '../product/product-color-variants/product-color-variants.repository';
import { ProductColorRepository } from '../product/product-color/product-color.repository';
import { ProductImageGalleryRepository } from '../product/product-image-gallery/product-image-gallery.repository';
import { ProductOfferVariantsRepository } from '../product/product-offer-variants/product-offer-variants.repository';
import { ProductReviewRepository } from '../product/product-review/product-review.repository';
import { ProductSaleVariantsRepository } from '../product/product-sale-variants/product-sale-variants.repository';
import { ProductSaleRepository } from '../product/product-sale/product-sale.repository';
import { ProductSizeVariantsRepository } from '../product/product-size-variants/product-size-variants.repository';
import { ProductSizeRepository } from '../product/product-size/product-size.repository';
import { Product } from '../product/product/entity/product.entity';
import { ProductsRepository } from '../product/product/product.repository';
import { SalesTaxCMSRepository } from '../sales-tax-cms/sales-tax-cms.repository';

@Injectable()
export class SiteFrontendService {
    constructor(
        private readonly blogRepository: BlogPostRepository,
        private readonly blogPageCmsRepo: BlogPageCmsRepository,
        private readonly aboutUsPageCmsRepo: AboutUsPageCmsRepository,
        private readonly shopPageCmsRepo: ShopPageCmsRepository,
        private readonly contactUsPageCmsRepo: ContactUsPageCmsRepository,
        private readonly footerCmsRepo: FooterCmsRepository,
        private readonly headerBannerCmsRepo: HeaderBannerCmsRepository,
        private readonly trendingSectionCmsRepo: TrendingSectionCmsRepository,
        private readonly faqPageCmsRepo: FaqPageCmsRepository,
        private readonly valuesSectionRepo: ValuesSectionRepository,
        private readonly faqRepo: FAQRepository,
        private readonly footerCopyrightSectionRepository: FooterCopyrightSectionRepository,
        private readonly refundPageCmsRepository: RefundPageCmsRepository,
        private readonly privacyPageCmsRepository: PrivacyPageCmsRepository,
        private readonly socialMediaLinkRepository: SocialMediaLinkRepository,
        private readonly accountDashboardCmsRepository: AccountDashboardCmsRepository,
        private readonly latestArrivalCmsRepository: LatestArrivalCmsRepository,
        private readonly looksYouWillLoveCmsRepository: LooksYouWillLoveCmsRepository,
        private readonly instagramSectionCmsRepository: InstagramSectionCmsRepository,
        private readonly latestNewsCmsRepository: LatestNewsCmsRepository,
        private readonly clearenceSaleSectionCmsRepository: ClearenceSaleSectionCmsRepository,
        private readonly latestCollectionSectionRepository: LatestCollectionSectionRepository,
        private readonly instagramServiceRepository: InstagramServiceRepository,
        private readonly billingAddressRepository: BillingAddressRepository,
        private readonly shippingAddressRepository: ShippingAddressRepository,
        private readonly productsRepository: ProductsRepository,
        private readonly productCategoryRepository: ProductCategoryRepository,
        private readonly productSizeRepository: ProductSizeRepository,
        private readonly productColorRepository: ProductColorRepository,
        private readonly productSaleRepository: ProductSaleRepository,
        private readonly productSizeVariantsRepository: ProductSizeVariantsRepository,
        private readonly productColorVariantsRepository: ProductColorVariantsRepository,
        private readonly productSaleVariantsRepository: ProductSaleVariantsRepository,
        private readonly productOfferVariantsRepository: ProductOfferVariantsRepository,
        private readonly productImageGalleryRepository: ProductImageGalleryRepository,
        private readonly productCareVariantsRepository: ProductCareVariantsRepository,
        private readonly productReviewRepository: ProductReviewRepository,
        private readonly salesTaxCMSRepository: SalesTaxCMSRepository,
    ) { }

    async getAllBlogs(page: number, limit: number): Promise<any | Error> {
        const blogs = await this.blogRepository.paginate(page, limit);
        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            blogs,
        );
    }

    async getBlogDetailsData(urlSlug: string): Promise<any | Error> {
        const blogData = await this.blogRepository.getLatestBlogListFromBeginning();

        if ((blogData as BlogPost[]).length === 0) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        const blogIndex = (blogData as BlogPost[]).findIndex(
            (blog) => blog.urlSlug === urlSlug,
        );
        if (blogIndex === -1) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        } else {
            const response = {
                blog: blogData[blogIndex],
                previousBlog:
                    blogIndex > 0
                        ? {
                            title: blogData[blogIndex - 1].title,
                            urlSlug: blogData[blogIndex - 1].urlSlug,
                        }
                        : null,
                nextBlog: blogData[blogIndex + 1]
                    ? {
                        title: blogData[blogIndex + 1].title,
                        urlSlug: blogData[blogIndex + 1].urlSlug,
                    }
                    : null,
            };

            return ResponseUtils.successResponseHandler(
                200,
                'Data fetched successfully!',
                'data',
                response,
            );
        }
    }

    async commonCMS(): Promise<any | Error> {
        const blogPageCmsData = await this.blogPageCmsRepo.findAll();
        const salesTaxCmsData = await this.salesTaxCMSRepository.findAll();
        const aboutUsPageCmsData = await this.aboutUsPageCmsRepo.findAll();
        const shopPageCmsData = await this.shopPageCmsRepo.findAll();
        const contactUsPageCmsData = await this.contactUsPageCmsRepo.findAll();
        const ourSocialMediaData = await this.socialMediaLinkRepository.findAll();
        const footerCmsData = await this.footerCmsRepo.findAll();
        const headerBannerCmsData = await this.headerBannerCmsRepo.findAll();
        const trendingSectionCmsData = await this.trendingSectionCmsRepo.findAll();
        const faqPageCmsData = await this.faqPageCmsRepo.findAll();
        const copyrightSectionData =
            await this.footerCopyrightSectionRepository.findByFilterQuery({});
        const refundPageCmsData = await this.refundPageCmsRepository.findAll();
        const privacyPageCmsData = await this.privacyPageCmsRepository.findAll();
        const accountDashboardCmsData =
            await this.accountDashboardCmsRepository.findAll();
        const latestArrivalCmsData =
            await this.latestArrivalCmsRepository.findAll();
        const looksYouWillLoveCmsData =
            await this.looksYouWillLoveCmsRepository.findAll();
        const instagramSectionCmsData =
            await this.instagramSectionCmsRepository.findAll();
        const latestNewsCmsData = await this.latestNewsCmsRepository.findAll();
        const latestNewsData = await this.blogRepository.getBlogList();
        const clearenceSaleSectionCmsData =
            await this.clearenceSaleSectionCmsRepository.findAll();
        const latestCollectionSectionData =
            await this.latestCollectionSectionRepository.getDataASC();
        const instagramSectionData =
            await this.instagramServiceRepository.findByFilterQuery({});
        const trendingCategories =
            await this.productCategoryRepository.findByFilterQuery({ isTrending: 1 });
        const latestProducts = await this.productsRepository.findLatestProducts(15);
        const popularProducts =
            (trendingCategories as ProductCategory[])?.length > 0
                ? await this.productsRepository.findPopularProducts(
                    trendingCategories as ProductCategory[],
                    25,
                )
                : [];
        const clearenceSale = await this.productSaleRepository.findByFilterQuery({
            name: 'Clearence Sale',
        });
        const clearenceSaleProducts = clearenceSale[0]?.id
            ? await this.productsRepository.findClearenceSaleProducts(
                clearenceSale[0]?.id,
                10,
            )
            : [];
        const categories =
            await this.productCategoryRepository.findAllWithProductCount();
        const sizes = await this.productSizeRepository.findByFilterQuery({});
        const colors = await this.productColorRepository.findByFilterQuery({});
        const sales = await this.productSaleRepository.findByFilterQuery({});

        const response = {
            blogPageCmsData,
            aboutUsPageCmsData,
            shopPageCmsData,
            contactUsPageCmsData,
            ourSocialMediaData,
            footerCmsData,
            headerBannerCmsData,
            trendingSectionCmsData,
            faqPageCmsData,
            copyrightSectionData,
            refundPageCmsData,
            privacyPageCmsData,
            accountDashboardCmsData,
            salesTaxCmsData,
            homePageCmsData: {
                trendingCategories,
                latestArrivalCmsData,
                latestProducts,
                looksYouWillLoveCmsData,
                popularProducts,
                clearenceSaleSectionCmsData,
                clearenceSaleProducts,
                instagramSectionCmsData,
                latestCollectionSectionData,
                instagramSectionData,
            },
            latestNewsCmsData,
            latestNewsData,
            shopPage: {
                categories,
                sizes,
                colors,
                sales,
            },
        };

        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            response,
        );
    }

    async aboutUsPage(): Promise<any | Error> {
        const valuesSectionData = await this.valuesSectionRepo.findByFilterQuery(
            {},
        );
        const faqData = await this.faqRepo.findByFilterQuery({});

        const response = {
            valuesSectionData,
            faqData,
        };

        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            response,
        );
    }

    async getUserAddresses(userId: string): Promise<any | Error> {
        const billingAddresses =
            await this.billingAddressRepository.findByFilterQuery({ userId: userId });
        const shippingAddresses =
            await this.shippingAddressRepository.findByFilterQuery({
                userId: userId,
            });
        const response = {
            billingAddresses,
            shippingAddresses,
        };

        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            response,
        );
    }

    async filterByProductName(
        limit: any,
        page: any,
        keyword: any,
        parentCategoryId: any,
        maxPrice: any,
        minPrice: any,
        color: any,
        size: any,
        sale: any,
        sortBy: any,
        readyToShip: any,
    ): Promise<any | Error> {
        const data = await this.productsRepository.filterByProductsName(
            limit,
            page,
            keyword,
            parentCategoryId,
            maxPrice,
            minPrice,
            color,
            size,
            sale,
            sortBy,
            readyToShip,
        );

        //(data);

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        return ResponseUtils.successResponseHandler(
            200,
            'Product Successfully fetched',
            'data',
            data,
        );
    }

    async findOneBySlug(slug: string): Promise<Product | Error> {
        const findBySlug = await this.productsRepository.findOneProductBySlug(slug);

        if (!findBySlug) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        const product = findBySlug as Product;
        const sizeVariants =
            await this.productSizeVariantsRepository.findByFilterQuery({
                productId: product.id,
            });
        const salesVariants =
            await this.productSaleVariantsRepository.findByFilterQuery({
                productId: product.id,
            });
        const colorVariants =
            await this.productColorVariantsRepository.findByFilterQuery({
                productId: product.id,
            });
        const offerVariants =
            await this.productOfferVariantsRepository.findOfferDetailsByProductId(
                product.id,
            );
        const careVariants =
            await this.productCareVariantsRepository.findWithImageByProductId(
                product.id,
            );
        const imageGallery =
            await this.productImageGalleryRepository.findByFilterQuery({
                productId: product.id,
            });
        const reviews = await this.productReviewRepository.findAllWithUserInfo(
            product.id,
        );
        const relatedProducts = await this.productsRepository.getRelatedProducts(
            product.id,
            product.parentCategoryId,
        );

        const data = {
            ...product,
            sizeVariants,
            colorVariants,
            salesVariants,
            offerVariants,
            careVariants,
            imageGallery,
            reviews,
            ...relatedProducts,
        };
        return ResponseUtils.successResponseHandler(
            200,
            'Product data fetched Successfully!',
            'data',
            data,
        );
    }
}
