import {Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import axios from 'axios';
import {pickupCreate, pickupRate, upsRate} from 'src/helper/ups-payload';
import {UpsConfigurationRepository} from 'src/modules/ups-configuration/ups-configuration.repository';

@Injectable()
export class UpsCoreService {
    constructor(
        @InjectRepository(UpsConfigurationRepository)
        private readonly repository: UpsConfigurationRepository
    ) {
    }

    async rate(dto: any) {
        try {
            let ratedata: any = [];
            const upsConfig = await this.repository.findAll();
            const username = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUserName : upsConfig[0].sandboxUserName;
            const password = upsConfig[0].credentialType == 'live' ? upsConfig[0].livePassword : upsConfig[0].sandboxPassword;
            const accessLicenseNumber = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveAccessLicenseNumber : upsConfig[0].sandboxAccessLicenseNumber;
            const baseUrl = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUrl : upsConfig[0].sandboxUrl;

            const config = {
                headers: {
                    'Content-Type': "application/json",
                    'AccessLicenseNumber': accessLicenseNumber,
                    'Username': username,
                    'Password': password,
                    'transId': "Neutrix2022",
                    'transactionSrc': "Neutrix"
                }
            };
            let service = {};
            if (dto.shippingAddress.stateCode == "MI") {
                service = {
                    "12": "3 Day Select",
                    "03": "Ground UPS",
                    "01": "Next Day Air",
                    "14": "Next Day Air Early AM",
                    "13": "Next Day Air Saver",
                }
            } else {
                service = {
                    "02": "2nd Day Air",
                    "59": "2nd Day Air A.M.",
                    "12": "3 Day Select",
                    "03": "Ground UPS",
                    "01": "Next Day Air",
                    "14": "Next Day Air Early AM",
                    "13": "Next Day Air Saver",
                }
            }
            for (var key of Object.keys(service)) {
                const payload = upsRate(
                    dto.shippingAddress.addressOne,
                    dto.shippingAddress.city,
                    dto.shippingAddress.stateOrProvinceCode,
                    dto.shippingAddress.postCode,
                    "US",
                    key,
                    service[key],
                    dto.weightValue,
                    dto.shippingAddress.name,
                    upsConfig[0]
                );

                const response = await axios.post(baseUrl + 'ship/v1/rating/Rate', payload, config);
                if (response.status === 200) {
                    ratedata.push(
                        {
                            serviceName: service[key],
                            serviceType: key,
                            price: parseFloat(response.data.RateResponse.RatedShipment.TotalCharges.MonetaryValue)
                        }
                    );
                }
            }
            return ratedata;
        } catch (e) {
            if (e.response) {
                
                return {
                    statusCode: e.response.status,
                    data: e.response.data
                }
            } else {
                throw e;
            }
        }
    }

    async pickupCreate(dto: any) {
        try {
            const upsConfig = await this.repository.findAll();
            const username = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUserName : upsConfig[0].sandboxUserName;
            const password = upsConfig[0].credentialType == 'live' ? upsConfig[0].livePassword : upsConfig[0].sandboxPassword;
            const accessLicenseNumber = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveAccessLicenseNumber : upsConfig[0].sandboxAccessLicenseNumber;
            const baseUrl = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUrl : upsConfig[0].sandboxUrl;

            const config = {
                headers: {
                    'Content-Type': "application/json",
                    'AccessLicenseNumber': accessLicenseNumber,
                    'Username': username,
                    'Password': password,
                }
            };

            const payload = await pickupCreate(
                dto.shippingAddress.addressOne,
                dto.shippingAddress.city,
                dto.shippingAddress.stateCode,
                dto.shippingAddress.postCode,
                "US",
                dto.serviceType,
                dto.serviceType,
                dto.weightValue,
                dto.shippingAddress.name,
                dto.shippingAddress.mobile,
                upsConfig[0]
            );

            const response = await axios.post(baseUrl + 'ship/v1/pickups', payload, config);
            if (response.status === 200) {
                return response.data
            }
        } catch (e) {
            if (e.response) {
                
                return {
                    statusCode: e.response.status,
                    data: e.response.data
                }
            } else {
                throw e;
            }
        }
    }

    async pickupRate(dto: any) {
        try {
            const upsConfig = await this.repository.findAll();
            const username = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUserName : upsConfig[0].sandboxUserName;
            const password = upsConfig[0].credentialType == 'live' ? upsConfig[0].livePassword : upsConfig[0].sandboxPassword;
            const accessLicenseNumber = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveAccessLicenseNumber : upsConfig[0].sandboxAccessLicenseNumber;
            const baseUrl = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUrl : upsConfig[0].sandboxUrl;

            const config = {
                headers: {
                    'Content-Type': "application/json",
                    'AccessLicenseNumber': accessLicenseNumber,
                    'Username': username,
                    'Password': password,
                }
            };

            const payload = await pickupRate(
                dto.pickupAddress.addressOne,
                dto.pickupAddress.city,
                dto.pickupAddress.stateCode,
                dto.pickupAddress.postCode,
                "US",
                upsConfig[0]
            );

            const response = await axios.post(baseUrl + 'ship/v1/pickups/rating', payload, config);
            if (response.status === 200) {
                return {
                    price: parseFloat(response.data.PickupRateResponse.RateResult.GrandTotalOfAllCharge)
                }
            }
        } catch (e) {
            if (e.response) {
                
                return {
                    statusCode: e.response.status,
                    data: e.response.data
                }
            } else {
                throw e;
            }
        }
    }
}
