import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from '@nestjs/common';
import {FileFieldsInterceptor} from '@nestjs/platform-express';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {getMulterConfig} from 'src/core/multer.config';
import {Constants} from 'src/utils/constants';
import {UpsertHeaderCmsDto} from './dto/upsert-header-cms.dto';
import {HeaderCmsService} from './header-cms.service';

@Controller({path: 'header-cms', version: Constants.API_VERSION_1})

export class HeaderCmsController {
    constructor(private readonly service: HeaderCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: 'logo', maxCount: 1}
        ],
        getMulterConfig(
            './uploads/header'
        )
    ))
    updateOrCreateData(
        @Body() dto: UpsertHeaderCmsDto,
        @UploadedFiles() files: {
            logo?: Express.Multer.File,
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
