import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateEmailKeywordDto} from "./dto/create-email-keyword.dto";
import {UpdateEmailKeywordDto} from "./dto/update-email-keyword.dto";
import {EmailKeyword} from "./entities/email-keyword.entity";

@EntityRepository(EmailKeyword)
export class EmailKeywordRepository extends Repository<EmailKeyword> {
    async createEntity(dto: CreateEmailKeywordDto): Promise<EmailKeyword | Error> {
        try {
            const {emailKeywordLabel, emailKeywordValue} = dto;
            const entity = this.create({
                emailKeywordLabel,
                emailKeywordValue
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<EmailKeyword | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateEmailKeywordDto): Promise<EmailKeyword | Error> {
        try {
            await this.update(id, {
                emailKeywordLabel: dto.emailKeywordLabel,
                emailKeywordValue: dto.emailKeywordValue
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<EmailKeyword[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(EmailKeyword)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}