import {EntityRepository, FindOperator, getConnection, getRepository, Repository} from "typeorm";
import {OrderDetails} from "./entity/order-details.entity";
import { Product } from "src/modules/product/product/entity/product.entity";

@EntityRepository(OrderDetails)
export class OrderDetailsRepository extends Repository<OrderDetails> {

    async createEntity(
        dto: any
    ): Promise<any | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findAllEntities(take: number, page: number, query: any): Promise<any | Error> {
        try {
            let strstartDate: any;
            let strendDate: any;
            type FindOption = {
                createdAt?: FindOperator<Date>,
                orderId?: FindOperator<string>,
            }
            // const findOptions: FindOption = {};
            // if(query.orderId) findOptions.orderId = query.orderId;
            if (query.startDate) strstartDate = query.startDate;
            if (query.endDate) strendDate = query.endDate;

            const startDate = new Date(strstartDate);
            startDate.setUTCHours(0, 0, 0, 0);
            const endDate = new Date(strendDate);
            endDate.setUTCHours(23, 59, 59, 999);

            let dataQuery = getRepository(OrderDetails)
                .createQueryBuilder("order-details")
                .select("order-details.orderId as orderId")
                .addSelect("order-details.name as product_name")
                .addSelect("IFNULL(p.cost,0) as cost")
                .addSelect("IFNULL(order-details.itemPrice,0) as itemPrice")
                .addSelect("IFNULL(order-details.quantity,0) as orderQuantity")
                .addSelect("(IFNULL(order-details.quantity,0)*IFNULL(p.cost,0)) as costTotal")
                .addSelect("IFNULL(order-details.itemTotal,0) as priceTotal")
                .addSelect("(IFNULL(order-details.itemTotal,0)-(IFNULL(order-details.quantity,0)*IFNULL(p.cost,0))) as profit")
                .addSelect("order-details.createdAt as createdAt")
                .leftJoin(Product, "p", "order-details.productId = p.id")
            if (query.startDate && query.endDate) {
                dataQuery.where('order-details.createdAt > :startDate', {startDate: new Date(startDate)})
                dataQuery.andWhere('order-details..createdAt < :endDate', {endDate: new Date(endDate)})
            }

            if (query.orderId) {
                dataQuery.where('order-details.orderId = :orderId', {orderId: query.orderId})
            }

            const data = await dataQuery.getRawMany();

            const totalOrders = await this.count({
                where: {isDeleted: false}
            });

            const responseData = {
                data: data,
                totalOrders: totalOrders,
            };

            return responseData;
        } catch (e) {
            return e;
        }
    }


    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(OrderDetails)
                .set({isDeleted: true})
                .where("orderId = :orderId", {orderId: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}