import {CoreEntity} from "src/core/core.entity";
import {AfterInsert, Column, Entity, getConnection, OneToMany} from "typeorm";
import {OrderDetails} from "../../order-details/entity/order-details.entity";

@Entity('orders')
export class Order extends CoreEntity {
    @Column({unique: true})
    orderId: string;

    @Column()
    customerId: string;

    @Column()
    customerName: string;

    @Column()
    productPriceTotal: string;

    @Column({default: null})
    couponNumber: string;

    @Column({default: null})
    couponDiscount: string;

    @Column({default: null})
    totalQuantity: string;

    @Column({default: null})
    itemTotalDiscount: string;

    @Column({default: null})
    subTotal: string;

    @Column({default: null})
    total: string;

    @Column({default: null})
    remarks: string;

    @Column({default: null})
    paymentMethodId: string;

    @Column({default: null, type: "longtext"})
    shippingAddress: string;

    @Column({default: null, type: "longtext"})
    billingAddress: string;

    @Column({default: "Cash on Delivery"})
    paymentMethodName: string;

    @Column({default: "PENDING"})
    paymentStatus: string;

    @Column({default: "PENDING"})
    status: string;

    @Column()
    createdBy: string;

    @Column({default: null})
    shippingCost: string;

    @Column({default: null})
    shippingType: string;

    @Column({default: null})
    serviceType: string;

    @Column({default: null})
    trackingNumber: string;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0, nullable: true })
    totalWeight: number;

    @Column({default: null})
    discountType: string;

    @Column({default: '0', nullable: true})
    discountAmount: string;

    @Column({default: '0', nullable: true})
    discountTotal: string;

    @Column({default: null, nullable: true})
    salesTaxTitle: string;

    @Column({type: "decimal", precision: 10, scale: 2, default: 0, nullable: true})
    salesTaxRate: string;

    @Column({type: "decimal", precision: 10, scale: 2, default: 0, nullable: true})
    salesTaxAmount: number;

    @Column({type: "decimal", precision: 10, scale: 2, default: 0, nullable: true})
    totalDiscount: number;

    @Column({default: '************1111', nullable: true})
    cardNumber: string;


    @Column({default: null, nullable: true})
    labelStockType: string;

    @Column({default: null, nullable: true})
    labelImageType: string;

    @Column({default: null, nullable: true, type: 'longtext'})
    labelImageEncode: string;


    // @OneToMany(() => InvoiceItem, invoiceItem => invoiceItem.invoice)
    // invoiceItem: InvoiceItem[];

    // @ManyToOne(() => User)
    // @JoinColumn({ name: 'customerId' })
    // user: User;

    @OneToMany(() => OrderDetails, orderDetails => orderDetails.order, {nullable: true})
    orders: OrderDetails[];

    @AfterInsert()
    processInvoiceService() {
        
        if (this.couponNumber != null) {
            let sql = "CALL cuponUsedWithInvoice('" + this.couponNumber + "')";
            
            getConnection().query(sql);
        }

    }

}
