import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {ProductCategoryService} from "./product-category.service";
import {CreateProductCategoryDto} from "./dto/create-product-category.dto";
import {UpdateProductCategoryDto} from "./dto/update-product-category.dto";
import {Constants} from "src/utils/constants";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";

@Controller({path: "product-category", version: Constants.API_VERSION_1})

export class ProductCategoryController {
    constructor(private readonly service: ProductCategoryService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "image", maxCount: 1},
        ]
    ))
    create(
        @Body() dto: CreateProductCategoryDto,
        @UploadedFiles() files: {
            image?: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch(":id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "image", maxCount: 1}
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: UpdateProductCategoryDto,
        @UploadedFiles() files: {
            image?: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    remove(@Param("id") id: string) {
        return this.service.remove(id);
    }
}
