import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {Constants} from "src/utils/constants";
import {CreateProductOfferDto} from "./dto/create-product-offer.dto";
import {UpdateProductOfferDto} from "./dto/update-product-offer.dto";
import {ProductOffer} from "./entities/product-offer.entity";
import {ProductOfferRepository} from "./product-offer.repository";
import {ResponseUtils} from "src/utils/response.utils";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class ProductOfferService {
    constructor(
        private readonly repository: ProductOfferRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateProductOfferDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<any | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "ProductOffer/");
            dto.image = image;
        } else dto.image = null;

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product offer created successfully", "data", data);
    }

    async findAll(): Promise<ProductOffer[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product offer list fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<any | Error> {
        let data: any = await this.repository.findOneEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product offer fetched successfully", "data", data);
    }

    async update(
        id: string,
        dto: UpdateProductOfferDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<ProductOffer | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "ProductOffer/");
            dto.image = image;
        }
        const data = await this.repository.updateProductOffer(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product offer updated successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product offer deleted successfully");
    }
}
