import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from '@nestjs/common';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { Constants } from 'src/utils/constants';
import { ProductPerksVariantsService } from './product-perks-variants.service';
import { CreateProductPerksVariantsDto } from './dto/create-product-perks-variants.dto';
import { UpdateProductPerksVariantsDto } from './dto/update-product-perks-variants.dto';

@Controller({ path: 'product-perks-variants', version: Constants.API_VERSION_1 })
export class ProductPerksVariantsController {
    constructor(private readonly service: ProductPerksVariantsService) {}

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateProductPerksVariantsDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateProductPerksVariantsDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
