import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductSizeGuideRepository } from './product-size-guide.repository';
import { CreateProductSizeGuideDto } from './dto/create-product-size-guide.dto';
import { UpdateProductSizeGuideDto } from './dto/update-product-size-guide.dto';
import { ProductSizeGuideEntity } from './entities/product-size-guide.entity';

@Injectable()
export class ProductSizeGuideService {
    constructor(
        @InjectRepository(ProductSizeGuideRepository)
        private readonly repository: ProductSizeGuideRepository
    ) { }

    async create(dto: CreateProductSizeGuideDto): Promise<ProductSizeGuideEntity | Error> {
        const prevData = await this.repository.findByFilterQuery({ categoryId: dto.categoryId });
        if((prevData as ProductSizeGuideEntity[])?.length) {
            throw new HttpException(
                "Product size guide already exists.",
                HttpStatus.BAD_REQUEST
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product size guide created successfully", "data", data);
    }

    async findAll(): Promise<ProductSizeGuideEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product size guides fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductSizeGuideDto): Promise<ProductSizeGuideEntity | Error> {
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product size guide updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductSizeGuideEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product size guide fetched successfully", "data", data);
    }

    async findOneByCategory(id: string): Promise<ProductSizeGuideEntity | Error> {
        const data = await this.repository.findOne({ where: { categoryId: id } });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product size guide fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product size guide deleted successfully");
    }
}
