import { Module } from '@nestjs/common';

import { MailerModule } from '@nestjs-modules/mailer';
import { HandlebarsAdapter } from '@nestjs-modules/mailer/dist/adapters/handlebars.adapter';
import { ConfigModule } from '@nestjs/config';
import { JwtModule } from '@nestjs/jwt';
import { ServeStaticModule } from '@nestjs/serve-static';
import { TypeOrmModule } from '@nestjs/typeorm';
import { join } from 'path';
import { AppController } from './app.controller';
import { AppService } from './app.service';
import JwtConfigService from './core/jwt/jwt-config.service';
import { AboutUsPageCmsModule } from './modules/CMS/about-us-page-cms/about-us-page-cms.module';
import { AccountDashboardCmsModule } from './modules/CMS/account-dashboard-cms/account-dashboard-cms.module';
import { BlogPageCmsModule } from './modules/CMS/blog-page-cms/blog-page-cms.module';
import { ClearenceSaleSectionCmsModule } from './modules/CMS/clearence-sale-section-cms/clearence-sale-section-cms.module';
import { ContactUsPageCmsModule } from './modules/CMS/contact-us-page-cms/contact-us-page-cms.module';
import { FaqPageCmsModule } from './modules/CMS/faq-page-cms/faq-page-cms.module';
import { FooterCmsModule } from './modules/CMS/footer-cms/footer-cms.module';
import { HeaderBannerCmsModule } from './modules/CMS/header-banner-cms/header-banner-cms.module';
import { HeaderCmsModule } from './modules/CMS/header-cms/header-cms.module';
import { InstagramSectionCmsModule } from './modules/CMS/instagram-section-cms/instagram-section-cms.module';
import { LatestArrivalCmsModule } from './modules/CMS/latest-arrival-cms/latest-arrival-cms.module';
import { LatestNewsCmsModule } from './modules/CMS/latest-news-cms/latest-news-cms.module';
import { LooksYouWillLoveCmsModule } from './modules/CMS/looks-you-will-love-cms/looks-you-will-love-cms.module';
import { PrivacyPageCmsModule } from './modules/CMS/privacy-page-cms/privacy-cms.module';
import { ProductDetailsPageCmsModule } from './modules/CMS/product-details-page-cms/product-details-page-cms.module';
import { RefundPageCmsModule } from './modules/CMS/refund-page-cms/refund-cms.module';
import { ShopPageCmsModule } from './modules/CMS/shop-page-cms/shop-page-cms.module';
import { TrendingSectionCmsModule } from './modules/CMS/trending-section-cms/trending-section-cms.module';
import { CollectionPageModule } from './modules/CRUD/collection-page/collection-page.module';
import { ContactModelModule } from './modules/CRUD/contact-model/contact-model.module';
import { FAQModule } from './modules/CRUD/faq/faq.module';
import { FooterCollectionSectionModule } from './modules/CRUD/footer-collection-section/footer-collection-section.module';
import { FooterCompanySectionModule } from './modules/CRUD/footer-company-section/footer-company-section.module';
import { FooterCopyrightSectionModule } from './modules/CRUD/footer-copyright-section/footer-copyright-section.module';
import { FooterHelpSectionModule } from './modules/CRUD/footer-help-section/footer-help-section.module';
import { FooterServiceSectionModule } from './modules/CRUD/footer-service-section/footer-service-section.module';
import { FooterSocialMediaSectionModule } from './modules/CRUD/footer-social-media-section/footer-social-media-section.module';
import { HeroSectionSliderModule } from './modules/CRUD/hero-section-slider/hero-section-slider.module';
import { InstagramServiceModule } from './modules/CRUD/instagram-service/instagram-service.module';
import { LatestCollectionSectionModule } from './modules/CRUD/latest-collection-section/latest-collection-section.module';
import { SocialMediaLinkModule } from './modules/CRUD/social-media-link/social-media-link.module';
import { TrendingSectionModule } from './modules/CRUD/trending-section/trending-section.module';
import { ValuesSectionModule } from './modules/CRUD/values-section/values-section.module';
import { DoSpacesServicerovider } from './modules/SpacesModule/SpacesService';
import { DoSpacesService } from './modules/SpacesModule/SpacesService/doSpacesService';
import { SpacesModule } from './modules/SpacesModule/spaces.module';
import { BillingAddressModule } from './modules/address/billing-address/billing-address.module';
import { ShippingAddressModule } from './modules/address/shipping-address/shipping-address.module';
import { AuthModule } from './modules/auth/auth.module';
import { BlogPostModule } from './modules/blog/blog-post/blog-post.module';
import { CardInformationModule } from './modules/card-information/card-information.module';
import { CouponModule } from './modules/coupon/coupon.module';
import { EmailKeywordModule } from './modules/email-keyword/email-keyword.module';
import { EmailNotifierRepository } from './modules/email-notifier/email-notifier-repository';
import { EmailNotifierModule } from './modules/email-notifier/email-notifier.module';
import { FileUploadsModule } from './modules/file-uploads/file-uploads.module';
import { FileUploadsService } from './modules/file-uploads/file-uploads.service';
import { GiftCardModule } from './modules/gift-card/gift-card.module';
import { MailSenderModule } from './modules/mail-sender/mail-sender.module';
import { MailSenderService } from './modules/mail-sender/mail-sender.service';
import { OrderDetailsModule } from './modules/orders/order-details/order-details.module';
import { OrdersModule } from './modules/orders/order/orders.module';
import { PaymentMethodModule } from './modules/orders/payment-method/payment-method.module';
import { PaymentConfigurationModule } from './modules/payment-configuration/payment-configuration.module';
import { PaymentGatewayModule } from './modules/payment-gateway/payment-gateway.module';
import { ProductCareVariantsModule } from './modules/product/product-care-variants/product-care-variants.module';
import { ProductCareModule } from './modules/product/product-care/product-care.module';
import { ProductCategoryModule } from './modules/product/product-category/product-category.module';
import { ProductColorVariantsModule } from './modules/product/product-color-variants/product-color-variants.module';
import { ProductColorModule } from './modules/product/product-color/product-color.module';
import { ProductDumpImportModule } from './modules/product/product-dump-import/product-dump-import.module';
import { ProductImageGalleryModule } from './modules/product/product-image-gallery/product-image-gallery.module';
import { ProductOfferVariantsModule } from './modules/product/product-offer-variants/product-offer-variants.module';
import { ProductOfferModule } from './modules/product/product-offer/product-offer.module';
import { ProductReviewModule } from './modules/product/product-review/product-review.module';
import { ProductSaleVariantsModule } from './modules/product/product-sale-variants/product-sale-variants.module';
import { ProductSaleModule } from './modules/product/product-sale/product-sale.module';
import { ProductSizeGuideModule } from './modules/product/product-size-guide/product-size-guide.module';
import { ProductSizeVariantsModule } from './modules/product/product-size-variants/product-size-variants.module';
import { ProductSizeModule } from './modules/product/product-size/product-size.module';
import { ProductModule } from './modules/product/product/product.module';
import { RefundItemModule } from './modules/refund-item/refund-item.module';
import { ReturnProductsModule } from './modules/return-products/return-products.module';
import { SalesTaxCMSModule } from './modules/sales-tax-cms/sales-tax-cms.module';
import { SalesTaxModule } from './modules/sales-tax/sales-tax.module';
import { SendEmailRepository } from './modules/send-email/send-email-repository';
import { SendEmailModule } from './modules/send-email/send-email.module';
import { SendEmailService } from './modules/send-email/send-email.service';
import { ShippingMethodCMSModule } from './modules/shipping-method-cms/shipping-method-cms.module';
import { ShippingMethodsModule } from './modules/shipping-method/shipping-method.module';
import { SiteFrontendModule } from './modules/site-frontend/site-frontend.module';
import { UpsConfigurationModule } from './modules/ups-configuration/ups-configuration.module';
import { UpsCoreModule } from './modules/ups/ups-core/ups-core.module';
import { UpsShippingModule } from './modules/ups/ups-shipping/ups-shipping.module';
import { UsersRepository } from './modules/users/user.repository';
import { UsersModule } from './modules/users/users.module';
import { UsersService } from './modules/users/users.service';
import { ProductPerksModule } from './modules/product/product-perks/product-perks.module';
import { DashboardModule } from './modules/dashboard/dashboard.module';
import { ProductPerksVariantsModule } from './modules/product/product-perks-variants/product-perks-variants.module';

@Module({
    imports: [
        ServeStaticModule.forRoot({
            rootPath: join(__dirname, '..', 'uploads'),
        }),
        ConfigModule.forRoot({ isGlobal: true }),
        TypeOrmModule.forRoot({
            type: 'mysql',
            host: process.env.HOST,
            port: parseInt(process.env.PORT),
            username: process.env.USERNAME,
            // username: 'root',
            password: process.env.PASSWORD,
            database: process.env.DATABASE,
            autoLoadEntities: true,
            synchronize: true,
            dropSchema: false,
        }),
        TypeOrmModule.forFeature([UsersRepository]),
        TypeOrmModule.forFeature([SendEmailRepository]),
        TypeOrmModule.forFeature([EmailNotifierRepository]),
        JwtModule.registerAsync({
            useClass: JwtConfigService,
        }),
        MailerModule.forRoot({
            transport: {
                host: process.env.EMAIL_HOST,
                port: parseInt(process.env.EMAIL_PORT),
                secure: true,
                auth: {
                    user: process.env.EMAIL_USER,
                    pass: process.env.EMAIL_PASS,
                },
            },
            defaults: {
                from: `"${process.env.EMAIL_SENDER_NAME}" <${process.env.EMAIL_SENDER_MAIL}>`,
            },
            template: {
                dir: __dirname + '/templates/**/*',
                adapter: new HandlebarsAdapter(),
                options: {
                    strict: true,
                },
            },
        }),
        UsersModule,
        AuthModule,
        EmailNotifierModule,
        FileUploadsModule,
        SendEmailModule,
        MailSenderModule,
        SpacesModule,
        PaymentConfigurationModule,
        PaymentGatewayModule,
        CardInformationModule,
        EmailKeywordModule,
        HeroSectionSliderModule,
        BlogPostModule,
        SiteFrontendModule,
        BlogPageCmsModule,
        AboutUsPageCmsModule,
        ShopPageCmsModule,
        ContactUsPageCmsModule,
        ValuesSectionModule,
        FAQModule,
        ContactModelModule,
        HeaderBannerCmsModule,
        HeaderCmsModule,
        FooterCmsModule,
        TrendingSectionCmsModule,
        FaqPageCmsModule,
        FooterCollectionSectionModule,
        FooterCompanySectionModule,
        FooterHelpSectionModule,
        FooterServiceSectionModule,
        FooterCopyrightSectionModule,
        FooterSocialMediaSectionModule,
        TrendingSectionModule,
        SocialMediaLinkModule,
        RefundPageCmsModule,
        PrivacyPageCmsModule,
        CollectionPageModule,
        AccountDashboardCmsModule,
        LatestArrivalCmsModule,
        LooksYouWillLoveCmsModule,
        InstagramSectionCmsModule,
        LatestNewsCmsModule,
        ClearenceSaleSectionCmsModule,
        LatestCollectionSectionModule,
        InstagramServiceModule,
        BillingAddressModule,
        ShippingAddressModule,
        ProductCategoryModule,
        ProductSizeModule,
        ProductSaleModule,
        ProductColorModule,
        ProductOfferModule,
        ProductCareModule,
        ProductColorVariantsModule,
        ProductSizeVariantsModule,
        ProductCareVariantsModule,
        ProductImageGalleryModule,
        ProductDetailsPageCmsModule,
        ProductOfferVariantsModule,
        ProductSaleVariantsModule,
        ProductModule,
        CouponModule,
        GiftCardModule,
        RefundItemModule,
        OrderDetailsModule,
        OrdersModule,
        ReturnProductsModule,
        ProductReviewModule,
        UpsCoreModule,
        UpsShippingModule,
        UpsConfigurationModule,
        SalesTaxModule,
        SalesTaxCMSModule,
        ShippingMethodsModule,
        ShippingMethodCMSModule,
        PaymentMethodModule,
        ProductDumpImportModule,
        ProductSizeGuideModule,
        ProductPerksModule,
        ProductPerksVariantsModule,
        DashboardModule,
    ],
    controllers: [AppController],
    providers: [
        AppService,
        UsersService,
        SendEmailService,
        FileUploadsService,
        DoSpacesService,
        DoSpacesServicerovider,
        MailSenderService,
    ],
})
export class AppModule { }
