import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {FooterCompanySectionRepository} from './footer-company-section.repository';
import {CreateFooterCompanySectionDto} from './dto/create-footer-company-section.dto';
import {UpdateFooterCompanySectionDto} from './dto/update-footer-company-section.dto';
import {FooterCompanySectionEntity} from './entity/footer-company-section.entity';

@Injectable()
export class FooterCompanySectionService {
    constructor(
        @InjectRepository(FooterCompanySectionRepository)
        private readonly repository: FooterCompanySectionRepository
    ) {
    }

    async create(dto: CreateFooterCompanySectionDto): Promise<FooterCompanySectionEntity | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
    }

    async findAll(): Promise<FooterCompanySectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async update(id: string, dto: UpdateFooterCompanySectionDto): Promise<FooterCompanySectionEntity | Error> {
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", data);
    }

    async findOne(id: string): Promise<FooterCompanySectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
