import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { InstagramServiceRepository } from "./instagram-service.repository";
import { CreateInstagramServiceDto } from "./dto/create-instagram-service.dto";
import { UpdateInstagramServiceDto } from "./dto/update-instagram-service.dto";
import { InstagramServiceEntity } from "./entities/instagram-service.entity";
import { DoSpacesService } from "../../SpacesModule/SpacesService/doSpacesService";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";

@Injectable()
export class InstagramServiceService {
  constructor(
    @InjectRepository(InstagramServiceRepository)
    private readonly repository: InstagramServiceRepository,
    private readonly doSpaceService: DoSpacesService
  ) {
  }

  async create(dto: CreateInstagramServiceDto,
    files: {
      media_url: UploadedMulterFileI
    }): Promise<InstagramServiceEntity | Error> {
    const checkDuplicate = await this.repository.findOne({
      where: {
        permalink: dto.permalink,
        isDeleted: false
      }
    });
    if (checkDuplicate) {
      throw new HttpException(
        "Instagram permalink already exists.",
        HttpStatus.BAD_REQUEST
      );
    }

    if (files && files.media_url) {
      const media_url: any = await this.doSpaceService.uploadFile(files.media_url[0], "InstagramService/");
      dto.media_url = media_url;
    }
    const data = await this.repository.createEntity(dto);
    if (!data) {
      throw new HttpException(
        Constants.CREATE_FAILED,
        HttpStatus.BAD_REQUEST
      );
    }
    return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
  }

  async findAll(): Promise<InstagramServiceEntity[] | Error> {
    const data = await this.repository.findByFilterQuery({});
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }

  async update(id: string, dto: UpdateInstagramServiceDto,
    files: {
      media_url: UploadedMulterFileI
    }): Promise<InstagramServiceEntity | Error> {
    const findOneData: any = await this.repository.findOneEntity(id);
    if (!findOneData) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    if (files && files.media_url) {
      const media_url: any = await this.doSpaceService.uploadFile(files.media_url[0], "InstagramService/");
      dto.media_url = media_url;
    }
    const data = await this.repository.updateEntity(id, dto);
    if (!data) {
      throw new HttpException(
        Constants.UPDATE_FAILED,
        HttpStatus.BAD_REQUEST
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
  }

  async findOne(id: string): Promise<InstagramServiceEntity[] | Error> {
    const data = await this.repository.findByFilterQuery({ id: id });
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }

  async remove(id: string): Promise<boolean | Error> {
    const data = await this.repository.deleteEntity(id);
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
  }
}
