import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from "@nestjs/common";
import {Constants} from "src/utils/constants";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {UploadedFiles, UseInterceptors} from "@nestjs/common/decorators";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";
import {CreateSocialMediaLinkDto} from "./dto/create-social-media-link.dto";
import {UpdateSocialMediaLinkDto} from "./dto/update-social-media-link.dto";
import {SocialMediaLinkService} from "./social-media-link.service";

@Controller({path: "social-media-link", version: Constants.API_VERSION_1})

export class SocialMediaLinkController {
    constructor(private readonly service: SocialMediaLinkService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "logo", maxCount: 1}
        ]
    ))
    create(
        @Body() dto: CreateSocialMediaLinkDto,
        @UploadedFiles() files: {
            logo: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "logo", maxCount: 1}
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: UpdateSocialMediaLinkDto,
        @UploadedFiles() files: {
            logo: UploadedMulterFileI
        }) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
