import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { urlSlug } from 'src/helper/common-helper';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { ProductCareRepository } from './product-care.repository';
import { CreateProductCareDto } from './dto/create-product-care.dto';
import { ProductCare } from './entities/product-care.entity';
import { UpdateProductCareDto } from './dto/update-product-care.dto';

@Injectable()
export class ProductCareService {
    constructor(
        @InjectRepository(ProductCareRepository)
        private readonly repository: ProductCareRepository,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async create(dto: CreateProductCareDto,
        files: {
            image?: UploadedMulterFileI
        }): Promise<ProductCare | Error> {
        const res: any = await this.repository.findByName(dto.name);
        if (res.length > 0) {
            throw new HttpException(
                "Product care name already exists.",
                HttpStatus.BAD_REQUEST
            );
        }

        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "ProductCare/");
            dto.image = image;
        } else dto.image = null;

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product care created successfully", "data", data);
    }

    async findAll(): Promise<ProductCare[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product care fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductCareDto,
        files: {
            image?: UploadedMulterFileI
        }): Promise<ProductCare | Error> {
        const category: any = await this.repository.findOneEntity(id);
        if (!category) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "ProductCare/");
            dto.image = image;
        } else dto.image = null;

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product care updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductCare[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product care fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product care deleted successfully");
    }
}
