import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductDumpImportDto } from "./dto/create-product-dump-import.dto";
import { UpdateProductDumpImportDto } from "./dto/update-product-dump-import.dto";
import { ProductDumpImport } from "./entity/product-dump-import.entity";


@EntityRepository(ProductDumpImport)
export class ProductDumpImportRepository extends Repository<ProductDumpImport> {

    async createEntity(
        dto: CreateProductDumpImportDto
    ): Promise<ProductDumpImport | Error> {
        try {
            const { DumpID, Name, Sku, ParentCategoryName, Cost, Price, DiscountPrice, Stock, Weight, OuterShell, Lining, Size, Color, Sale } = dto;
            const entity = this.create({ DumpID, Name, Sku, ParentCategoryName, Cost, Price, DiscountPrice, Stock, Weight, OuterShell, Lining, Size, Color, Sale });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductDumpImport[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(
        id: string
    ): Promise<ProductDumpImport | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }


    async updateEntity(
        id: string,
        dto: UpdateProductDumpImportDto
    ): Promise<ProductDumpImport | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductDumpImport)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

}