import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductImageGalleryDto } from "./dto/create-product-image-gallery.dto";
import { UpdateProductImageGalleryDto } from "./dto/update-product-image-gallery.dto";
import { ProductImageGallery } from "./entities/product-image-gallery.entity";

@EntityRepository(ProductImageGallery)
export class ProductImageGalleryRepository extends Repository<ProductImageGallery> {

    async createEntity(dto: CreateProductImageGalleryDto): Promise<ProductImageGallery | Error> {
        try {
            const { productId, productName, image } = dto;
            const entity = this.create({
                productId,
                productName,
                image
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductImageGallery[] | Error> {
       
       
       
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductImageGallery | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductImageGalleryDto): Promise<ProductImageGallery | Error> {
        try {
            await this.update(id, {
                productId: dto.productId,
                productName: dto.productName,
                image: dto.image
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductImageGallery)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    // async findByName(name: string): Promise<ProductImageGallery[] | Error> {
    //     try {
    //         return await this.find({
    //             where: { 
    //                 name: name, 
    //                 isDeleted: false 
    //             }
    //         });
    //     }
    //     catch (e) {
    //         return e;
    //     }
    // }

    async deleteImage(images : any): Promise<any | Error> {
        try {
            await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductImageGallery)
                .where("image IN (:image)", { image: images })
                .execute();
        }
        catch (e) {
            return e;
        }
    }

    async deleteImageByProductId(id : any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
            .createQueryBuilder()
            .delete()
            .from(ProductImageGallery)
            .where("productId = :productId", { productId : id })
            .execute();
        if(deleteResult.affected > 0) {
            return true;
        }
        return false;
        }
        catch (e) {
            return e;
        }
    }
}