import { EntityRepository, getConnection, Repository } from "typeorm";
import { ProductPerksVariantsEntity } from "./entities/product-perks-variants.entity";
import { UpdateProductPerksVariantsDto } from "./dto/update-product-perks-variants.dto";
import { CreateProductPerksVariantsDto } from "./dto/create-product-perks-variants.dto";

@EntityRepository(ProductPerksVariantsEntity)
export class ProductPerksVariantsRepository extends Repository<ProductPerksVariantsEntity> {

    async createEntity(dto: CreateProductPerksVariantsDto): Promise<ProductPerksVariantsEntity | Error> {
        try {
            const { productId, productName, perkId, perkTitle } = dto;
            const entity = this.create({
                productId,
                productName,
                perkId,
                perkTitle
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<any | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductPerksVariantsEntity | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductPerksVariantsDto): Promise<ProductPerksVariantsEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductPerksVariantsEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    // async findByName(name: string): Promise<ProductSizeVariants[] | Error> {
    //     try {
    //         return await this.find({
    //             where: { 
    //                 name: name, 
    //                 isDeleted: false 
    //             }
    //         });
    //     }
    //     catch (e) {
    //         return e;
    //     }
    // }

    async deletePerkVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductPerksVariantsEntity)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertPerkVariant(newPerk: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductPerksVariantsEntity)
                .values(newPerk)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductPerksValues(id: any, newPerkEntity: any): Promise<any | Error> {
        try {
            const deleteCurrent = await this.deletePerkVariant(id);
            await this.bulkInsertPerkVariant(newPerkEntity)
        } catch (e) {
            return e;
        }
    }
}