import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { ProductPerksEntity } from "./entities/product-perks.entity";
import { CreateProductPerksDto } from "./dto/create-product-perks.dto";
import { UpdateProductPerksDto } from "./dto/update-product-perks.dto";
@EntityRepository(ProductPerksEntity)
export class ProductPerksRepository extends Repository<ProductPerksEntity> {

    async createEntity(dto: CreateProductPerksDto): Promise<ProductPerksEntity | Error> {
        try {
            const { title, description, image } = dto;
            const entity = this.create({
                title,
                description,
                image,
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductPerksEntity[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductPerksEntity | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductPerksDto): Promise<ProductPerksEntity | Error> {
        try {
            await this.update(id, {
                image: dto.image,
                title: dto.title,
                description: dto.description,
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductPerksEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductPerksEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    title: name,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }
}