import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductPerksRepository } from './product-perks.repository';
import { ProductPerksEntity } from './entities/product-perks.entity';
import { CreateProductPerksDto } from './dto/create-product-perks.dto';
import { UpdateProductPerksDto } from './dto/update-product-perks.dto';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
@Injectable()
export class ProductPerksService {
    constructor(
        @InjectRepository(ProductPerksRepository)
        private readonly repository: ProductPerksRepository,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async create(dto: CreateProductPerksDto,
        files: {
            image?: UploadedMulterFileI
        }): Promise<ProductPerksEntity | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "ProductPerk/");
            dto.image = image;
        } else dto.image = null;
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product Perk created successfully", "data", data);
    }

    async findAll(): Promise<ProductPerksEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Perk fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductPerksDto,
        files: {
            image?: UploadedMulterFileI
        }): Promise<ProductPerksEntity | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "ProductPerk/");
            dto.image = image;
        } else dto.image = null;

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Perk updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductPerksEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Perk fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product Perk deleted successfully");
    }
}
