import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductSizeGuideDto } from "./dto/create-product-size-guide.dto";
import { UpdateProductSizeGuideDto } from "./dto/update-product-size-guide.dto";
import { ProductSizeGuideEntity } from "./entities/product-size-guide.entity";

@EntityRepository(ProductSizeGuideEntity)
export class ProductSizeGuideRepository extends Repository<ProductSizeGuideEntity> {

    async createEntity(dto: CreateProductSizeGuideDto): Promise<ProductSizeGuideEntity | Error> {
        try {
            const { categoryId, categoryName, guide } = dto;
            const entity = this.create({
                categoryId, categoryName, guide
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductSizeGuideEntity[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductSizeGuideEntity | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductSizeGuideDto): Promise<ProductSizeGuideEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductSizeGuideEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(categoryName: string): Promise<ProductSizeGuideEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    categoryName: categoryName,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }
}