import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {ProductSizeService} from "./product-size.service";
import {CreateProductSizeDto} from "./dto/create-product-size.dto";
import {UpdateProductSizeDto} from "./dto/update-product-size.dto";
import {Constants} from "src/utils/constants";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";

@Controller({path: "product-size", version: Constants.API_VERSION_1})

export class ProductSizeController {
    constructor(private readonly service: ProductSizeService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(
        @Body() dto: CreateProductSizeDto
    ) {
        return this.service.create(dto);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch(":id")
    @UseGuards(AdminAuthGuard)
    update(
        @Param("id") id: string,
        @Body() dto: UpdateProductSizeDto,
    ) {
        return this.service.update(id, dto);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    remove(@Param("id") id: string) {
        return this.service.remove(id);
    }
}
