import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CreateProductStockImportDto } from './dto/create-product-stock-import.dto';
import { UpdateProductStockImportDto } from './dto/update-product-stock-import.dto';
import { ProductStockImport } from './entity/product-stock-import.entity';
import { ProductStockImportRepository } from './product-stock-import.repository';

@Injectable()
export class ProductStockImportService {

    constructor(
        @InjectRepository(ProductStockImportRepository)
        private readonly repository: ProductStockImportRepository,
        private readonly fileService: FileUploadsService,
    ) { }

    async create(dto: CreateProductStockImportDto): Promise<ProductStockImport | Error> {

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data created","data", data);
    }

    async findAll(): Promise<ProductStockImport[] | Error> {
      
        const data = await this.repository.findByFilterQuery({});
        
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data fetched","data", data);
    }

    async update(id: string, dto: UpdateProductStockImportDto): Promise<ProductStockImport | Error> {

        const dataPrevious : any = await this.repository.findOneEntity(id);
        if(!dataPrevious){
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data updated","data", data);
    }

    async findOne(id: string): Promise<ProductStockImport[] | Error> {
      
        const data = await this.repository.findByFilterQuery({id: id});
        
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.successResponseHandler(200,"Weight data fetched","data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
    
        return ResponseUtils.buildDeletedData(
                data,
        );
    }


}
