import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from '@nestjs/common';
import {Constants} from 'src/utils/constants';
import {SalesTaxService} from './sales-tax.service';
import {CreateSalesTaxDto} from './dto/create-sales-tax.dto';
import {UpdateSalesTaxDto} from './dto/update-sales-tax.dto';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';

@Controller({path: 'sales-tax', version: Constants.API_VERSION_1})

export class SalesTaxController {
    constructor(private readonly service: SalesTaxService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateSalesTaxDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get('/state/:stateName')
    findByStateName(@Param('stateName') stateName: string) {
        return this.service.findByStateName(stateName);
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/:id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateSalesTaxDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
