import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {UpdateOrCreateUpsConfigurationDto} from './dto/update-create-ups-configuration.dto';
import {UpsConfiguration} from './entities/ups-configuration.entity';
import {UpsConfigurationRepository} from './ups-configuration.repository';

@Injectable()
export class UpsConfigurationService {
    constructor(
        @InjectRepository(UpsConfigurationRepository)
        private readonly repository: UpsConfigurationRepository
    ) {
    }

    async updateOrCreateData(
        dto: UpdateOrCreateUpsConfigurationDto
    ): Promise<UpsConfiguration | Error> {
        const isExists = await this.repository.find();
        if (Object.keys(isExists).length === 0) {
            if (dto.credentialType == "sandbox") {
                const payload = {
                    sandboxAccessLicenseNumber: dto.accessLicenseNumber,
                    sandboxUrl: dto.url,
                    sandboxUserName: dto.userName,
                    sandboxPassword: dto.password,
                    addressLine: dto.addressLine,
                    shipperAccountName: dto.shipperAccountName,
                    shipperAccountNumber: dto.shipperAccountNumber,
                    city: dto.city,
                    stateProvienceCode: dto.stateProvienceCode,
                    postalCode: dto.postalCode,
                    countryCode: dto.countryCode,
                    country: dto.country,
                    credentialType: dto.credentialType
                }
                const data = await this.repository.createEntity(payload);
                if (!data) {
                    throw new HttpException(
                        Constants.GENERIC_ERROR,
                        HttpStatus.FORBIDDEN
                    );
                }
                return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
            }
            if (dto.credentialType == "live") {
                const payload = {
                    liveAccessLicenseNumber: dto.accessLicenseNumber,
                    liveUrl: dto.url,
                    liveUserName: dto.userName,
                    livePassword: dto.password,
                    addressLine: dto.addressLine,
                    shipperAccountName: dto.shipperAccountName,
                    shipperAccountNumber: dto.shipperAccountNumber,
                    city: dto.city,
                    stateProvienceCode: dto.stateProvienceCode,
                    postalCode: dto.postalCode,
                    countryCode: dto.countryCode,
                    country: dto.country,
                    credentialType: dto.credentialType
                }
                const data = await this.repository.createEntity(payload);
                if (!data) {
                    throw new HttpException(
                        Constants.GENERIC_ERROR,
                        HttpStatus.FORBIDDEN
                    );
                }
                return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
            }
        } else {
            let id = isExists[0].id;
            if (dto.credentialType == "sandbox") {
                const payload = {
                    sandboxAccessLicenseNumber: dto.accessLicenseNumber,
                    sandboxUrl: dto.url,
                    sandboxUserName: dto.userName,
                    sandboxPassword: dto.password,
                    addressLine: dto.addressLine,
                    shipperAccountName: dto.shipperAccountName,
                    shipperAccountNumber: dto.shipperAccountNumber,
                    city: dto.city,
                    stateProvienceCode: dto.stateProvienceCode,
                    postalCode: dto.postalCode,
                    countryCode: dto.countryCode,
                    country: dto.country,
                    credentialType: dto.credentialType
                }
                const data = await this.repository.updateEntity(payload, id);
                if (!data) {
                    throw new HttpException(
                        Constants.UPDATE_FAILED,
                        HttpStatus.FORBIDDEN
                    );
                }
                return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
            }
            if (dto.credentialType == "live") {
                const payload = {
                    liveAccessLicenseNumber: dto.accessLicenseNumber,
                    liveUrl: dto.url,
                    liveUserName: dto.userName,
                    livePassword: dto.password,
                    addressLine: dto.addressLine,
                    shipperAccountName: dto.shipperAccountName,
                    shipperAccountNumber: dto.shipperAccountNumber,
                    city: dto.city,
                    stateProvienceCode: dto.stateProvienceCode,
                    postalCode: dto.postalCode,
                    countryCode: dto.countryCode,
                    country: dto.country,
                    credentialType: dto.credentialType
                }
                const data = await this.repository.updateEntity(payload, id);
                if (!data) {
                    throw new HttpException(
                        Constants.UPDATE_FAILED,
                        HttpStatus.FORBIDDEN
                    );
                }
                return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
            }
        }
    }

    async findAll(): Promise<UpsConfiguration[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
