import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {FileUploadsService} from 'src/modules/file-uploads/file-uploads.service';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {UpsertHeaderCmsDto} from './dto/upsert-header-cms.dto';
import {HeaderCms} from './entities/header-cms.entity';
import {HeaderCmsRepository} from './header-cms.repository';

@Injectable()
export class HeaderCmsService {
    constructor(
        @InjectRepository(HeaderCmsRepository)
        private readonly repository: HeaderCmsRepository,
        private readonly fileService: FileUploadsService
    ) {
    }

    async updateOrCreateData(
        dto: UpsertHeaderCmsDto,
        files: { logo?: Express.Multer.File }
    ): Promise<HeaderCms | Error> {
        if (files && files.logo) {
            dto.logo = await this.fileService.saveFile(files.logo[0], []);
        }
        const isExists = await this.repository.find();
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.GENERIC_ERROR,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
        }
    }

    async findAll(): Promise<HeaderCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
