import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors,
} from '@nestjs/common';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { Constants } from 'src/utils/constants';
import { UpdateShopPageCmsDto } from './dto/update-shop-page-cms.dto';
import { ShopPageCmsService } from './shop-page-cms.service';

@Controller({ path: 'shop-page-cms', version: Constants.API_VERSION_1 })
export class ShopPageCmsController {
    constructor(private readonly service: ShopPageCmsService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(
        FileFieldsInterceptor([{ name: 'topBannerImage', maxCount: 1 }]),
    )
    updateOrCreateData(
        @Body() dto: UpdateShopPageCmsDto,
        @UploadedFiles()
        files: {
            topBannerImage: UploadedMulterFileI;
        },
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
