import { EntityRepository, Repository } from 'typeorm';
import { UpdateShopPageCmsDto } from './dto/update-shop-page-cms.dto';
import { ShopPageCmsEntity } from './entities/shop-page-cms.entity';

@EntityRepository(ShopPageCmsEntity)
export class ShopPageCmsRepository extends Repository<ShopPageCmsEntity> {
    async createEntity(
        dto: UpdateShopPageCmsDto,
    ): Promise<ShopPageCmsEntity | Error> {
        try {
            const entity = this.create({
                pageTitle: dto.pageTitle,
                sectionTitle: dto.sectionTitle,
                sectionHeader: dto.sectionHeader,
                buttonText: dto.buttonText,
                buttonLink: dto.buttonLink,
                topBannerImage: dto.topBannerImage,
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(
        dto: UpdateShopPageCmsDto,
        id: string,
    ): Promise<ShopPageCmsEntity | Error> {
        try {
            await this.update(id, {
                pageTitle: dto.pageTitle,
                sectionTitle: dto.sectionTitle,
                sectionHeader: dto.sectionHeader,
                buttonText: dto.buttonText,
                buttonLink: dto.buttonLink,
                topBannerImage: dto.topBannerImage,
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<ShopPageCmsEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false,
                },
            });
        } catch (e) {
            return e;
        }
    }
}
