import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateHeroSectionSliderDto} from "./dto/create-hero-section-slider.dto";
import {HeroSectionSliderEntity} from "./entities/hero-section-slider.entity";
import {UpdateHeroSectionSliderDto} from "./dto/update-hero-section-slider.dto";

@EntityRepository(HeroSectionSliderEntity)
export class HeroSectionSliderRepository extends Repository<HeroSectionSliderEntity> {
    async createEntity(dto: CreateHeroSectionSliderDto): Promise<HeroSectionSliderEntity | Error> {
        try {
            const {
                sectionHeader,
                sectionTitle,
                sectionSubTitle,
                buttonText,
                buttonLink,
                image
            } = dto;
            const entity = this.create({
                sectionHeader,
                sectionTitle,
                sectionSubTitle,
                buttonText,
                buttonLink,
                image
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<HeroSectionSliderEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<HeroSectionSliderEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(HeroSectionSliderEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateHeroSectionSliderDto): Promise<HeroSectionSliderEntity | Error> {
        try {
            await this.update(id, {
                sectionHeader: dto.sectionHeader,
                sectionTitle: dto.sectionTitle,
                sectionSubTitle: dto.sectionSubTitle,
                buttonText: dto.buttonText,
                buttonLink: dto.buttonLink,
                image: dto.image,
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }
}