import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from "@nestjs/common";
import {Constants} from "src/utils/constants";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {UploadedFiles, UseInterceptors} from "@nestjs/common/decorators";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";
import { CreateLatestCollectionSectionDto } from "./dto/create-latest-collection-section.dto";
import { UpdateLatestCollectionSectionDto } from "./dto/update-latest-collection-section.dto";
import { LatestCollectionSectionService } from "./latest-collection-section.service";

@Controller({path: "latest-collection-section", version: Constants.API_VERSION_1})

export class LatestCollectionSectionController {
    constructor(private readonly service: LatestCollectionSectionService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "image", maxCount: 1}
        ]
    ))
    create(
        @Body() dto: CreateLatestCollectionSectionDto,
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "image", maxCount: 1}
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: UpdateLatestCollectionSectionDto,
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
