import {EntityRepository, getConnection, Repository} from "typeorm";
import { LatestCollectionSectionEntity } from "./entities/latest-collection-section.entity";
import { CreateLatestCollectionSectionDto } from "./dto/create-latest-collection-section.dto";
import { UpdateLatestCollectionSectionDto } from "./dto/update-latest-collection-section.dto";

@EntityRepository(LatestCollectionSectionEntity)
export class LatestCollectionSectionRepository extends Repository<LatestCollectionSectionEntity> {
    async createEntity(dto: CreateLatestCollectionSectionDto): Promise<LatestCollectionSectionEntity | Error> {
        try {
            const {
                promotionalTitle,
                title,
                buttonText,
                buttonLink,
                image
            } = dto;
            const entity = this.create({
                promotionalTitle,
                title,
                buttonText,
                buttonLink,
                image
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<LatestCollectionSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<LatestCollectionSectionEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "ASC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async getDataASC(): Promise<LatestCollectionSectionEntity[] | Error> {
        try {
            return await this.find({
                order: {
                    createdAt: "ASC"
                },
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(LatestCollectionSectionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateLatestCollectionSectionDto): Promise<LatestCollectionSectionEntity | Error> {
        try {
            await this.update(id, {
                promotionalTitle: dto.promotionalTitle,
                title: dto.title,
                buttonText: dto.buttonText,
                buttonLink: dto.buttonLink,
                image: dto.image,
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }
}