import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ShippingAddressRepository } from './shipping-address.repository';
import { CreateShippingAddressDto } from './dto/create-shipping-address.dto';
import { UpdateShippingAddressDto } from './dto/update-shipping-address.dto';
import { ShippingAddress } from './entities/shipping-address.entity';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';

@Injectable()
export class ShippingAddressService {
    constructor(
        @InjectRepository(ShippingAddressRepository)
        private readonly repository: ShippingAddressRepository,
        private readonly spaceService: DoSpacesService
    ) { }

    async create(
        dto: CreateShippingAddressDto,
        request: any
    ): Promise<ShippingAddress | Error> {
        dto.userId = request.user.id;
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Shipping Address Created Successfully", "data", data);
    }

    async update(
        id: string,
        dto: UpdateShippingAddressDto,
        request: any
    ): Promise<ShippingAddress | Error> {
        const res : any = await this.repository.findOneEntity(id);
        if(!res){
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        dto.userId = request.user.id;
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Shipping address Updated successfully", "data", data);
    }

    async findAll(): Promise<ShippingAddress[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Shipping address data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<ShippingAddress | Error> {
        const data = await this.repository.findByFilterQuery({userId: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Shipping address data fetched successfully", "data", data[0]);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Shipping address data deleted successfully");
    }
}
