import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {CardInformationRepository} from './card-information.repository';
import {CreateCardInformationDto} from './dto/create-card-information.dto';
import {UpdateCardInformationDto} from './dto/update-card-information.dto';
import {CardInformation} from './entities/card-information.entity';

@Injectable()
export class CardInformationService {
    constructor(
        @InjectRepository(CardInformationRepository)
        private readonly repository: CardInformationRepository
    ) {
    }

    async create(dto: CreateCardInformationDto, request: any): Promise<CardInformation | Error> {
        const res: any = await this.repository.findOne({
            ccNum: dto.ccNum,
            userId: request.user.id,
            userType: request.user.userType,
            isDeleted: false
        });
        if (res) {
            throw new HttpException(
                "Card Number already exists.",
                HttpStatus.BAD_REQUEST
            );
        }

        const cardInfo = await this.repository.findByFilterQuery({
            userId: request.user.id,
            userType: request.user.userType
        });

        if (Object.keys(cardInfo).length === 0) {
            dto.isDefault = true;
        } else dto.isDefault = false;

        dto.userId = request.user.id;
        dto.userType = request.user.userType;

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(
            200,
            "CardInformation Data Created Successful!",
            "data",
            data);
    }

    async findAll(request: any): Promise<CardInformation[] | Error> {

        const data = await this.repository.findByFilterQuery({
            userId: request.user.id,
            userType: request.user.userType
        });

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "CardInformation Data Fetched", "data", data);
    }

    async update(id: string, dto: UpdateCardInformationDto, request: any): Promise<CardInformation | Error> {

        const CardInformation: any = await this.repository.findOneEntity(id);
        if (!CardInformation) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = await this.repository.updateCardInformation(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(
            200,
            "CardInformation Data Updated Successful!",
            "data",
            data);
    }

    async makeDefault(id: string, request: any): Promise<CardInformation | Error> {

        const cardInfo = await this.repository.findByFilterQuery({
            userId: request.user.id,
            userType: request.user.userType
        });
        for (let i = 0; i < Object.keys(cardInfo).length; i++) {
            if (cardInfo[i]) {
                await this.repository.update(cardInfo[i].id, {isDefault: false})
            }
        }

        const CardInformation: any = await this.repository.findOneEntity(id);
        if (!CardInformation) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = await this.repository.updateCardInformation(id, {isDefault: true});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(
            200,
            "CardInformation Data Updated Successful!",
            "data",
            data);
    }

    async findOne(id: string): Promise<CardInformation[] | Error> {

        const data = await this.repository.findByFilterQuery({id: id});

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "CardInformation Data Fetched", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(
            data,
            200,
            "CardInformation Data Deleted Successful!"
        );
    }
}
