import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {CreateEmailKeywordDto} from './dto/create-email-keyword.dto';
import {UpdateEmailKeywordDto} from './dto/update-email-keyword.dto';
import {EmailKeywordRepository} from './email-keyword.repository';
import {EmailKeyword} from './entities/email-keyword.entity';

@Injectable()
export class EmailKeywordService {
    constructor(
        @InjectRepository(EmailKeywordRepository)
        private readonly repository: EmailKeywordRepository
    ) {
    }

    async create(dto: CreateEmailKeywordDto): Promise<EmailKeyword | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(id: string, dto: UpdateEmailKeywordDto): Promise<EmailKeyword | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<EmailKeyword[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<EmailKeyword[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
