import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateGiftCardDto } from "./dto/create-gift-card.dto";
import { UpdateGiftCardDto } from "./dto/update-gift-card.dto";
import { GiftCard } from "./entities/gift-card.entity";

@EntityRepository(GiftCard)
export class GiftCardRepository extends Repository<GiftCard> {
    async createEntity(dto: CreateGiftCardDto): Promise<GiftCard | Error> {
        try {
            const entity = this.create({
                name: dto.name,
                description: dto.description,
                image: dto.image,
                price: dto.price,
                cost: dto.cost,
                warranty: dto.warranty,
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<GiftCard | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateGiftCardDto): Promise<GiftCard | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                description: dto.description,
                image: dto.image,
                price: dto.price,
                cost: dto.cost,
                warranty: dto.warranty,
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<GiftCard[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(GiftCard)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async getGiftCardsByQuery(giftCardIds: any): Promise<GiftCard[] | Error> {
        try {
            const giftCards = await getConnection()
                .createQueryBuilder()
                .from(GiftCard, "gift_card")
                .where("gift_card.id IN (:...ids)", { ids: giftCardIds })
                .execute();

            return giftCards;
        }
        catch (e) {
            return e;
        }
    }
}