import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { ProductOfferVariants } from "./entities/product-offer-variants.entity";
import { CreateProductOfferVariantsDto } from "./dto/create-product-offer-variants.dto";
import { UpdateProductOfferVariantsDto } from "./dto/update-product-offer-variants.dto";

@EntityRepository(ProductOfferVariants)
export class ProductOfferVariantsRepository extends Repository<ProductOfferVariants> {

    async createEntity(dto: CreateProductOfferVariantsDto): Promise<ProductOfferVariants | Error> {
        try {
            const { productId, productName, offerId, offerName } = dto;
            const entity = this.create({
                productId,
                productName,
                offerId,
                offerName
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductOfferVariants[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductOfferVariants | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductOfferVariantsDto): Promise<ProductOfferVariants | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductOfferVariants)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductOfferVariants[] | Error> {
        try {
            return await this.find({
                where: {
                    offerName: name,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOfferDetailsByProductId(productId: string): Promise<ProductOfferVariants[] | Error> {
        try {
          const sql = `SELECT pov.*, po.title as offerTitle, po.description offerDescription, po.image as offerImage FROM 
          product_offer_variants pov LEFT JOIN product_offer po 
          ON pov.offerId = po.id WHERE pov.productId = '${productId}';`;
          const entityManager = getManager();
          return entityManager.query(sql);
        } catch (e) {
          return e;
        }
      }

    async deleteOfferVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductOfferVariants)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertOfferVariant(newOffer: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductOfferVariants)
                .values(newOffer)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductOfferValues(id: any, newOfferEntity: any): Promise<any | Error> {
        try {
            const deleteCurrent = await this.deleteOfferVariant(id);
            await this.bulkInsertOfferVariant(newOfferEntity)
        } catch (e) {
            return e;
        }
    }
}