import { Body, Controller, Delete, Get, Param, Patch, Post, UploadedFiles, UseGuards, UseInterceptors } from '@nestjs/common';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { Constants } from 'src/utils/constants';
import { CreateProductPerksDto } from './dto/create-product-perks.dto';
import { UpdateProductPerksDto } from './dto/update-product-perks.dto';
import { ProductPerksService } from './product-perks.service';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Controller({ path: 'product-perks', version: Constants.API_VERSION_1 })
export class ProductPerksController {
    constructor(private readonly service: ProductPerksService) {}

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "image", maxCount: 1 },
        ]
    ))
    create(@Body() dto: CreateProductPerksDto,
    @UploadedFiles() files: {
        image?: UploadedMulterFileI
    }) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "image", maxCount: 1 },
        ]
    ))
    update(@Param('id') id: string, @Body() dto: UpdateProductPerksDto,
    @UploadedFiles() files: {
        image?: UploadedMulterFileI
    }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
