import { EntityRepository, getConnection, Repository } from "typeorm";
import { ProductSale } from "./entities/product-sale.entity";
import { CreateProductSaleDto } from "./dto/create-product-sale.dto";
import { UpdateProductSaleDto } from "./dto/update-product-sale.dto";

@EntityRepository(ProductSale)
export class ProductSaleRepository extends Repository<ProductSale> {

    async createEntity(dto: CreateProductSaleDto): Promise<ProductSale | Error> {
        try {
            const { name, urlSlug } = dto;
            const entity = this.create({
                name,
                urlSlug
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductSale[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductSale | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductSaleDto): Promise<ProductSale | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                urlSlug: dto.urlSlug
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductSale)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductSale[] | Error> {
        try {
            return await this.find({
                where: {
                    name: name,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }
}