import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductSizeVariantsRepository } from './product-size-variants.repository';
import { CreateProductSizeVariantsDto } from './dto/create-product-size-variants.dto';
import { UpdateProductSizeVariantsDto } from './dto/update-product-size-variants.dto';
import { ProductSizeVariants } from './entities/product-size-variants.entity';

@Injectable()
export class ProductSizeVariantsService {
    constructor(
        @InjectRepository(ProductSizeVariantsRepository)
        private readonly repository: ProductSizeVariantsRepository
    ) { }

    async create(dto: CreateProductSizeVariantsDto): Promise<ProductSizeVariants | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product Size Variants created successfully", "data", data);
    }

    async findAll(): Promise<ProductSizeVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Size Variants fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductSizeVariantsDto): Promise<ProductSizeVariants | Error> {
        const category: any = await this.repository.findOneEntity(id);
        if (!category) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Size Variants updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductSizeVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Size Variants fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product Size Variants deleted successfully");
    }
}
