import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { generateStockId, isArraySame, urlSlug } from 'src/helper/common-helper';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CreateImportProductDto } from './dto/create-import-product.dto';
import { CreateProductDto } from './dto/create-product.dto';
import { UpdateProductDto } from './dto/update-product.dto';
import { Product } from './entity/product.entity';
import { ProductsRepository } from './product.repository';
import { v4 as uuidv4 } from 'uuid';
import readXlsxFile from 'read-excel-file/node';
import { CreateConfirmImportProductDto } from './dto/create-confirm-import-product.dto';
import { CreateConfirmStockInProductDto } from './dto/create-confirm-stockin-product.dto';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { ProductColorRepository } from '../product-color/product-color.repository';
import { ProductColorVariantsRepository } from '../product-color-variants/product-color-variants.repository';
import { CreateProductColorVariantsDto } from '../product-color-variants/dto/create-product-color-variants.dto';
import { ProductSizeRepository } from '../product-size/product-size.repository';
import { ProductSizeVariantsRepository } from '../product-size-variants/product-size-variants.repository';
import { CreateProductSizeVariantsDto } from '../product-size-variants/dto/create-product-size-variants.dto';
import { ProductImageGalleryRepository } from '../product-image-gallery/product-image-gallery.repository';
import { CreateProductImageGalleryDto } from '../product-image-gallery/dto/create-product-image-gallery.dto';
import { ProductCategoryRepository } from '../product-category/product-category.repository';
import { ProductSaleRepository } from '../product-sale/product-sale.repository';
import { CreateProductSaleVariantsDto } from '../product-sale-variants/dto/create-product-sale-variants.dto';
import { ProductSaleVariantsRepository } from '../product-sale-variants/product-sale-variants.repository';
import { ProductOfferRepository } from '../product-offer/product-offer.repository';
import { ProductCareRepository } from '../product-care/product-care.repository';
import { ProductCareVariantsRepository } from '../product-care-variants/product-care-variants.repository';
import { CreateProductCareVariantsDto } from '../product-care-variants/dto/create-product-care-variants.dto';
import { ProductOfferVariantsRepository } from '../product-offer-variants/product-offer-variants.repository';
import { CreateProductOfferVariantsDto } from '../product-offer-variants/dto/create-product-offer-variants.dto';
import { ProductSizeVariants } from '../product-size-variants/entities/product-size-variants.entity';
import { ProductCategory } from '../product-category/entities/product-category.entity';
import { CreateProductColorDto } from '../product-color/dto/create-product-color.dto';
import { CreateProductDumpImportDto } from '../product-dump-import/dto/create-product-dump-import.dto';
import { ProductDumpImportRepository } from '../product-dump-import/product-dump-importrepository';
import { CreateProductSizeDto } from '../product-size/dto/create-product-size.dto';
import { CreateProductSaleDto } from '../product-sale/dto/create-product-sale.dto';
import { ProductStockImportRepository } from '../product-stock-import/product-stock-import.repository';
import { CreateProductStockImportDto } from '../product-stock-import/dto/create-product-stock-import.dto';
import { CreateProductPerksVariantsDto } from '../product-perks-variants/dto/create-product-perks-variants.dto';
import { ProductPerksVariantsRepository } from '../product-perks-variants/product-perks-variants.repository';
import { ProductPerksRepository } from '../product-perks/product-perks.repository';

@Injectable()
export class ProductService {

    constructor(
        @InjectRepository(ProductsRepository)
        private readonly repository: ProductsRepository,
        @InjectRepository(ProductCategoryRepository)
        private readonly productCategoryRepository: ProductCategoryRepository,
        private readonly doSpaceService: DoSpacesService,
        @InjectRepository(ProductColorRepository)
        private readonly productColorRepository: ProductColorRepository,
        @InjectRepository(ProductColorVariantsRepository)
        private readonly productColorVariantsRepo: ProductColorVariantsRepository,
        @InjectRepository(ProductSizeRepository)
        private readonly productSizeRepo: ProductSizeRepository,
        @InjectRepository(ProductSizeVariantsRepository)
        private readonly productSizeVariantsRepo: ProductSizeVariantsRepository,
        @InjectRepository(ProductImageGalleryRepository)
        private readonly productImageGalleryRepo: ProductImageGalleryRepository,
        @InjectRepository(ProductSaleRepository)
        private readonly productSaleRepository: ProductSaleRepository,
        @InjectRepository(ProductSaleVariantsRepository)
        private readonly productSaleVariantsRepository: ProductSaleVariantsRepository,
        @InjectRepository(ProductOfferRepository)
        private readonly productOfferRepository: ProductOfferRepository,
        @InjectRepository(ProductOfferVariantsRepository)
        private readonly productOfferVariantsRepository: ProductOfferVariantsRepository,
        @InjectRepository(ProductCareRepository)
        private readonly productCareRepository: ProductCareRepository,
        @InjectRepository(ProductCareVariantsRepository)
        private readonly productCareVariantsRepository: ProductCareVariantsRepository,
        @InjectRepository(ProductDumpImportRepository)
        private readonly productDumpImportRepository: ProductDumpImportRepository,
        @InjectRepository(ProductStockImportRepository)
        private readonly productStockImportRepository: ProductStockImportRepository,
        @InjectRepository(ProductPerksRepository)
        private readonly productPerksRepository: ProductPerksRepository,
        @InjectRepository(ProductPerksVariantsRepository)
        private readonly productPerksVariantsRepository: ProductPerksVariantsRepository,
    ) {
    }

    private cleanNameString(stringName: string) {
        return stringName.trim().replace(/[^a-zA-Z0-9 ]/g, '');
    }

    async create(dto: CreateProductDto, files: {
        productImages?: UploadedMulterFileI[],
        featuredImage?: UploadedMulterFileI,
    }): Promise<any> {
        // start check parent category exist or not
        const productParentCatData = await this.productCategoryRepository.findOneEntity(
            dto.parentCategoryId
        );

        if (!productParentCatData) {
            throw new HttpException(
                "Category not found",
                HttpStatus.NOT_FOUND
            );
        }
        // end check parent category exist or not

        // take category name from parentCategoryId
        dto.parentCategoryName = (productParentCatData as ProductCategory).name;
        // take category name from parentCategoryId

        // start process featured image and upload it cloud storage
        if (files && files.featuredImage) {

            const featuredImage: any = await this.doSpaceService.uploadFile(files.featuredImage[0], 'Product/');
            dto.featuredImage = featuredImage;
        } else dto.featuredImage = 'https://golden-gal.nyc3.cdn.digitaloceanspaces.com/default_cms.png';
        // end process featured image and upload it cloud storage
        //

        let slug = urlSlug(dto.name);

        // Create a product entity
        const productEntity = {
            name: dto.name,
            featuredImage: dto.featuredImage,
            price: dto.price,
            discountPrice: dto.discountPrice,
            shortDescription: dto.shortDescription,
            longDescription: dto.longDescription,
            outerShell: dto.outerShell,
            lining: dto.lining,
            sku: dto.sku,
            cost: dto.cost,
            stock: dto.stock,
            isDeactive: dto.isDeactive ? dto.isDeactive : 0,
            isReadyToShip: dto.isReadyToShip ? dto.isReadyToShip : 0,
            parentCategoryId: dto.parentCategoryId,
            parentCategoryName: dto.parentCategoryName,
            weight: dto.weight,
            urlSlug: slug,
        }
        const product = await this.repository.createEntity(productEntity as CreateProductDto);
        // end of product create 

        // processing product image
        if (files && files.productImages) {
            for (const productFile of files.productImages) {

                const productImageUrl: any = await this.doSpaceService.uploadFile(productFile, 'Product/');
                const productImage = {
                    productId: product.id,
                    productName: product.name,
                    image: productImageUrl,
                }
                await this.productImageGalleryRepo.createEntity(productImage as CreateProductImageGalleryDto)
            }
        }
        // end of product image processiong 

        // processiong color and create it into color variants
        if (dto.color && dto.color.length > 0) {
            const colors = dto.color;
            for (const color of colors) {
                const colorData = await this.productColorRepository.findByName(color);

                if (!colorData) {

                } else {
                    if (Object.keys(colorData).length > 0) {
                        const colorVariant = {
                            productId: product.id,
                            productName: product.name,
                            colorId: colorData[0]['id'],
                            colorName: color,
                            colorCode: colorData[0]['colorCode']
                        }
                        await this.productColorVariantsRepo.createEntity(colorVariant as CreateProductColorVariantsDto);
                    } else {

                    }
                }
            }
        }
        // end of product color loop 


        // processiong size and create it into size variants
        if (dto.size && dto.size.length > 0) {
            const sizes = dto.size;
            for (const size of sizes) {

                const sizeData = await this.productSizeRepo.findByName(size);



                if (!sizeData) {

                } else {
                    if (Object.keys(sizeData).length > 0) {
                        const sizeVariant = {
                            productId: product.id,
                            productName: product.name,
                            sizeId: sizeData[0]['id'],
                            sizeName: size
                        }
                        await this.productSizeVariantsRepo.createEntity(sizeVariant as CreateProductSizeVariantsDto)
                    } else {

                    }
                }
            }
        }
        // end of product size process

        // start processiong product sales
        if (dto.sales && dto.sales.length > 0) {
            const sales = dto.sales;
            for (const sale of sales) {

                const saleData = await this.productSaleRepository.findByName(sale);

                if (!saleData) {

                } else {
                    if (Object.keys(saleData).length > 0) {
                        const saleVariant = {
                            productId: product.id,
                            productName: product.name,
                            saleId: saleData[0]['id'],
                            saleName: sale
                        }
                        await this.productSaleVariantsRepository.createEntity(saleVariant as CreateProductSaleVariantsDto)
                    } else {

                    }
                }
            }
        }
        // end processiong product sales

        // processiong offer and create it into offer variants
        if (dto.offers && dto.offers.length > 0) {
            const offers = dto.offers;
            for (const offer of offers) {

                const offerData = await this.productOfferRepository.findByName(offer);

                if (!offerData) {

                } else {
                    if (Object.keys(offerData).length > 0) {
                        const offerVariant = {
                            productId: product.id,
                            productName: product.name,
                            offerId: offerData[0]['id'],
                            offerName: offer
                        }
                        await this.productOfferVariantsRepository.createEntity(offerVariant as CreateProductOfferVariantsDto)
                    } else {

                    }
                }
            }
        }
        // end of product offer process

        //Staring add product care detail 
        if (dto.cares && dto.cares.length > 0) {
            const cares = dto.cares;
            for (const care of cares) {

                const careData = await this.productCareRepository.findByName(care);

                if (!careData) {

                } else {
                    if (Object.keys(careData).length > 0) {
                        const careVariant = {
                            productId: product.id,
                            productName: product.name,
                            careId: careData[0]['id'],
                            careName: care
                        }
                        await this.productCareVariantsRepository.createEntity(careVariant as CreateProductCareVariantsDto)
                    } else {

                    }
                }
            }
        }
        //Staring end product care detail 

        //Staring add product perk detail 
        if (dto.perks && dto.perks.length > 0) {
            const perks = dto.perks;
            for (const perk of perks) {

                const perkData = await this.productPerksRepository.findByName(perk);

                if (!perkData) {

                } else {
                    if (Object.keys(perkData).length > 0) {
                        const perkVariant = {
                            productId: product.id,
                            productName: product.name,
                            perkId: perkData[0]['id'],
                            perkTitle: perk
                        }
                        await this.productPerksVariantsRepository.createEntity(perkVariant as CreateProductPerksVariantsDto)
                    } else {

                    }
                }
            }
        }
        //Staring end product perk detail 

        const sizeVariants = await this.productSizeVariantsRepo.findByFilterQuery({ productId: product.id });
        const salesVariants = await this.productSaleVariantsRepository.findByFilterQuery({ productId: product.id });
        const colorVariants = await this.productColorVariantsRepo.findByFilterQuery({ productId: product.id });
        const offerVariants = await this.productOfferVariantsRepository.findByFilterQuery({ productId: product.id });
        const perksVariants = await this.productPerksVariantsRepository.findByFilterQuery({ productId: product.id });
        const careVariants = await this.productCareVariantsRepository.findByFilterQuery({ productId: product.id });
        const imageGallery = await this.productImageGalleryRepo.findByFilterQuery({ productId: product.id });



        const data = {
            ...product,
            sizeVariants,
            colorVariants,
            salesVariants,
            offerVariants,
            perksVariants,
            careVariants,
            imageGallery
        }

        return ResponseUtils.successResponseHandler(200, "Product created successfully!", "data", data);
    }

    // async findAll(page: number, limit: number): Promise<any | Error> {
    //     const data = await this.repository.paginate(page, limit);
    //     return ResponseUtils.successResponseHandler(200, "Products fetched successfully", "data", data);
    // }

    async findAll(page: number, limit: number): Promise<any | Error> {
        const data = await this.repository.findAllEntities(page, limit);
        return ResponseUtils.successResponseHandler(200, "Products fetched successfully", "data", data);
    }

    async getAllForAdmin(): Promise<any | Error> {
        const data = await this.repository.getAllForAdmin();
        return ResponseUtils.successResponseHandler(200, "Products fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<Product | Error> {
        const productSearch = await this.repository.findOneRawEntity(id);
        if (!productSearch) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const product = productSearch as Product;
        const sizeVariants = await this.productSizeVariantsRepo.findByFilterQuery({ productId: product.id });
        const salesVariants = await this.productSaleVariantsRepository.findByFilterQuery({ productId: product.id });
        const colorVariants = await this.productColorVariantsRepo.findByFilterQuery({ productId: product.id });
        const offerVariants = await this.productOfferVariantsRepository.findByFilterQuery({ productId: product.id });
        const perksVariants = await this.productPerksVariantsRepository.findByFilterQuery({ productId: product.id });
        const careVariants = await this.productCareVariantsRepository.findByFilterQuery({ productId: product.id });
        const imageGallery = await this.productImageGalleryRepo.findByFilterQuery({ productId: product.id });

        const data = {
            ...product,
            sizeVariants,
            colorVariants,
            salesVariants,
            offerVariants,
            perksVariants,
            careVariants,
            imageGallery
        }
        return ResponseUtils.successResponseHandler(200, "Product data fetched Successfully!", "data", data);
    }

    async findOneBySlug(slug: string): Promise<Product | Error> {



        const findBySlug = await this.repository.findOneProductBySlug(slug);

        if (!findBySlug) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const product = findBySlug as Product;
        const sizeVariants = await this.productSizeVariantsRepo.findByFilterQuery({ productId: product.id });
        const salesVariants = await this.productSaleVariantsRepository.findByFilterQuery({ productId: product.id });
        const colorVariants = await this.productColorVariantsRepo.findByFilterQuery({ productId: product.id });
        const offerVariants = await this.productOfferVariantsRepository.findByFilterQuery({ productId: product.id });
        const perksVariants = await this.productPerksVariantsRepository.findByFilterQuery({ productId: product.id });
        const careVariants = await this.productCareVariantsRepository.findByFilterQuery({ productId: product.id });
        const imageGallery = await this.productImageGalleryRepo.findByFilterQuery({ productId: product.id });

        const data = {
            ...product,
            sizeVariants,
            colorVariants,
            salesVariants,
            offerVariants,
            perksVariants,
            careVariants,
            imageGallery
        }
        return ResponseUtils.successResponseHandler(200, "Product data fetched Successfully!", "data", data);
    }

    async updateOneProduct(
        id: string,
        dto: UpdateProductDto,
        files: {
            productImages?: UploadedMulterFileI[],
            featuredImage?: UploadedMulterFileI,
        }
    ): Promise<Product | Error> {
        // start check parent category exist or not
        const productParentCatData = await this.productCategoryRepository.findOneEntity(
            dto.parentCategoryId
        );
        if (!productParentCatData) {
            throw new HttpException(
                "Category not found",
                HttpStatus.NOT_FOUND
            );
        }
        // end check parent category exist or not

        // take category name from parentCategoryId
        dto.parentCategoryName = (productParentCatData as ProductCategory).name;
        // take category name from parentCategoryId


        if (files && files.featuredImage) {
            const featuredImage: any = await this.doSpaceService.uploadFile(files.featuredImage[0], 'Product/');
            dto.featuredImage = featuredImage;
        }

        const productObj = await this.repository.findOneRawEntity(id);
        if (!productObj) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        let slug = urlSlug(dto.name);

        const productEntity = {
            name: dto.name,
            featuredImage: dto.featuredImage,
            price: dto.price,
            discountPrice: dto.discountPrice,
            shortDescription: dto.shortDescription,
            longDescription: dto.longDescription,
            outerShell: dto.outerShell,
            lining: dto.lining,
            sku: dto.sku,
            cost: dto.cost,
            stock: dto.stock,
            isDeactive: dto.isDeactive ? dto.isDeactive : 0,
            isReadyToShip: dto.isReadyToShip ? dto.isReadyToShip : 0,
            parentCategoryId: dto.parentCategoryId,
            parentCategoryName: dto.parentCategoryName,
            weight: dto.weight,
            urlSlug: slug,
        }
        const product = await this.repository.updateEntity(id, productEntity);

        // working with product size
        const currentSizes = await this.productSizeVariantsRepo.findByFilterQuery({ productId: id });

        if (dto.size && dto.size.length > 0) {
            const arrayCheck = await isArraySame(currentSizes, dto.size);
            if (!arrayCheck) {
                const newSizeEntity: any = [];
                for (const size of dto.size) {
                    const sizeData = await this.productSizeRepo.findByName(size);
                    if (!sizeData) {

                    } else {
                        if (Object.keys(sizeData).length > 0) {
                            newSizeEntity.push({
                                productId: id,
                                productName: productObj.name,
                                sizeId: sizeData[0]['id'],
                                sizeName: size
                            });
                        }
                    }
                }
                await this.productSizeVariantsRepo.updateProductSizeValues(id, newSizeEntity)
            }
        }
        // end of product size process 

        // processing color 
        const currentColor = await this.productColorVariantsRepo.findByFilterQuery({ productId: id });
        if (dto.color && dto.color.length > 0) {
            const arrayCheck = await isArraySame(currentColor, dto.color);
            if (!arrayCheck) {
                const newColorEntity: any = [];
                for (const color of dto.color) {
                    const colorData = await this.productColorRepository.findByName(color);
                    if (!colorData) {

                    } else {
                        if (Object.keys(colorData).length > 0) {
                            newColorEntity.push({
                                productId: id,
                                productName: productObj.name,
                                colorId: colorData[0]['id'],
                                colorName: color,
                                colorCode: colorData[0]['colorCode']
                            })
                        }
                    }
                }
                await this.productColorVariantsRepo.updateProductColorValues(id, newColorEntity)
            }
        }
        // end of processing color 

        // processing sales 
        const currentSale = await this.productSaleVariantsRepository.findByFilterQuery({ productId: id });
        if (dto.sales && dto.sales.length > 0) {
            const arrayCheck = await isArraySame(currentSale, dto.sales);
            if (!arrayCheck) {
                const newSaleEntity: any = [];
                for (const sale of dto.sales) {
                    // (dto.sales);
                    const saleData = await this.productSaleRepository.findByName(sale);
                    if (!saleData) {
                    } else {
                        if (Object.keys(saleData).length > 0) {
                            newSaleEntity.push({
                                productId: id,
                                productName: productObj.name,
                                saleId: saleData[0]['id'],
                                saleName: sale,
                            })
                        }
                    }
                }
                (newSaleEntity)
                await this.productSaleVariantsRepository.updateProductSaleValues(id, newSaleEntity)
            }
        }
        // end of processing sales 

        // processing offers
        const currentOffers = await this.productOfferVariantsRepository.findByFilterQuery({ productId: id });
        if (dto.offers && dto.offers.length > 0) {
            const arrayCheck = await isArraySame(currentOffers, dto.offers);
            if (!arrayCheck) {
                const newOffersEntity: any = [];
                for (const offer of dto.offers) {
                    const offerData = await this.productOfferRepository.findByName(offer);
                    if (!offerData) {

                    } else {
                        if (Object.keys(offerData).length > 0) {
                            newOffersEntity.push({
                                productId: id,
                                productName: productObj.name,
                                offerId: offerData[0]['id'],
                                offerName: offer,
                            })
                        }
                    }
                }
                await this.productOfferVariantsRepository.updateProductOfferValues(id, newOffersEntity)
            }
        }
        // end of processing offers

        // processing cares
        const currentCares = await this.productCareVariantsRepository.findByFilterQuery({ productId: id });
        if (dto.cares && dto.cares.length > 0) {
            const arrayCheck = await isArraySame(currentCares, dto.cares);
            if (!arrayCheck) {
                const newCaresEntity: any = [];
                for (const care of dto.cares) {
                    const careData = await this.productCareRepository.findByName(care);
                    if (!careData) {

                    } else {
                        if (Object.keys(careData).length > 0) {
                            newCaresEntity.push({
                                productId: id,
                                productName: productObj.name,
                                careId: careData[0]['id'],
                                careName: care,
                            })
                        }
                    }
                }
                await this.productCareVariantsRepository.updateProductCareValues(id, newCaresEntity)
            }
        }
        // end of processing cares

        // processing perks
        const currentPerks = await this.productPerksVariantsRepository.findByFilterQuery({ productId: id });
        if (dto.perks && dto.perks.length > 0) {
            const arrayCheck = await isArraySame(currentPerks, dto.perks);
            if (!arrayCheck) {
                const newPerkEntity: any = [];
                for (const perk of dto.perks) {
                    const perkData = await this.productPerksRepository.findByName(perk);
                    if (!perkData) {

                    } else {
                        if (Object.keys(perkData).length > 0) {
                            newPerkEntity.push({
                                productId: id,
                                productName: productObj.name,
                                perkId: perkData[0]['id'],
                                perkTitle: perk,
                            })
                        }
                    }
                }
                await this.productPerksVariantsRepository.updateProductPerksValues(id, newPerkEntity);
            }
        }
        // end of processing perks

        if (files && files.productImages) {
            if (files && files.productImages) {
                for (const productFile of files.productImages) {
                    const productImageUrl: any = await this.doSpaceService.uploadFile(productFile, 'Product/');
                    const productImage = {
                        productId: id,
                        productName: productObj.name,
                        image: productImageUrl,
                    }
                    await this.productImageGalleryRepo.createEntity(productImage as CreateProductImageGalleryDto)
                }
            }
        }

        if (dto.deletedImage && dto.deletedImage.length > 0) {
            await this.productImageGalleryRepo.deleteImage(dto.deletedImage);
        }

        const updatedProduct = product as Product;

        const sizeVariants = await this.productSizeVariantsRepo.findByFilterQuery({ productId: updatedProduct.id });
        const salesVariants = await this.productSaleVariantsRepository.findByFilterQuery({ productId: updatedProduct.id });
        const colorVariants = await this.productColorVariantsRepo.findByFilterQuery({ productId: updatedProduct.id });
        const offerVariants = await this.productOfferVariantsRepository.findByFilterQuery({ productId: updatedProduct.id });
        const perksVariants = await this.productPerksVariantsRepository.findByFilterQuery({ productId: updatedProduct.id });
        const careVariants = await this.productCareVariantsRepository.findByFilterQuery({ productId: updatedProduct.id });
        const imageGallery = await this.productImageGalleryRepo.findByFilterQuery({ productId: updatedProduct.id });

        const data = {
            ...updatedProduct,
            sizeVariants,
            colorVariants,
            salesVariants,
            offerVariants,
            perksVariants,
            careVariants,
            imageGallery
        }

        return ResponseUtils.successResponseHandler(200, "Product updated successfully!", "product", data);
    }

    async updateProductStock(dto: UpdateProductDto, productId: string): Promise<Product | Error> {
        const productEntity = {
            name: dto.name,
            price: dto.price,
            stock: dto.stock,
            weight: dto.weight,
            sku: dto.sku,
        }
        // size: dto.size,
        //     color: dto.color,
        const product = await this.repository.updateEntity(productId, productEntity);
        return product;
    }

    async updateProductRating(dto: UpdateProductDto, productId: string): Promise<Product | Error> {
        const productEntity = {
            averageRating: dto.averageRating,
            reviewCount: dto.reviewCount,
        }
        const product = await this.repository.updateEntity(productId, productEntity);
        return product;
    }

    async remove(id: string): Promise<any | Error> {
        const data = await this.repository.deleteEntity(id);

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(
            data,
            200,
            "Product data deleted successfully!"
        );
    }

    async importProductService(dto: CreateImportProductDto, file: Express.Multer.File): Promise<any> {
        let dumpID = uuidv4();
        const excelDataRow = await readXlsxFile(file.path).then((rows) => {
            let excelDataRowDump = [];

            rows.forEach(async element => {

                if (element[0] != "Name") {
                    let productRawData = {
                        'DumpID': dumpID,
                        'Name': element[0],
                        'Sku': element[1],
                        'ParentCategoryName': element[2],
                        'Cost': element[3],
                        'Price': element[4],
                        'DiscountPrice': element[5],
                        'Stock': element[6],
                        'Weight': element[7],
                        'OuterShell': element[8],
                        'Lining': element[9],
                        'Size': element[10],
                        'SizeIDs': [],
                        'Color': element[11],
                        'ColorIDs': [],
                        'Sale': element[12],
                        'SaleIDs': [],
                    };
                    excelDataRowDump.push(productRawData);
                }
            });

            return excelDataRowDump;
        })

        for (let index = 0; index < excelDataRow.length; index++) {
            const rowsData = excelDataRow[index];

            //Colors Section Start
            let colorsArray = [];
            if (JSON.stringify(rowsData.Color).length > 0) {
                let colorsData = JSON.stringify(rowsData.Color).trim();
                colorsArray = colorsData.split(',')


            }

            for (let j = 0; j < colorsArray.length; j++) {
                const colorsName = colorsArray[j];

                let colorsRawData = await this.productColorRepository.findByFilterQuery({
                    name: this.cleanNameString(colorsName).trim()
                });

                if (!colorsRawData) {
                    let clrDto = {
                        name: this.cleanNameString(colorsName).trim(),
                        colorCode: '#0000'
                    };
                    let newColorData = await this.productColorRepository.createEntity(clrDto as CreateProductColorDto)
                    excelDataRow[index].ColorIDs.push(newColorData);
                } else {
                    if (Object.keys(colorsRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(colorsName).trim(),
                            colorCode: '#0000'
                        };
                        let newColorData = await this.productColorRepository.createEntity(clrDto as CreateProductColorDto)
                        excelDataRow[index].ColorIDs.push(newColorData);
                    } else {
                        excelDataRow[index].ColorIDs.push(colorsRawData[0]);
                    }

                }

            }
            //Colors Section End


            // //Size Section Start
            let sizeArray = [];
            if (JSON.stringify(rowsData.Size).length > 0) {
                let sizeData = JSON.stringify(rowsData.Size).trim();
                sizeArray = sizeData.split(',')
            }

            for (let j = 0; j < sizeArray.length; j++) {
                const sizeName = sizeArray[j];

                let sizeRawData = await this.productSizeRepo.findByFilterQuery({
                    name: this.cleanNameString(sizeName.trim())
                });

                if (!sizeRawData) {
                    let sizeDto = {
                        name: this.cleanNameString(sizeName.trim()),

                    };
                    let newSizeData = await this.productSizeRepo.createEntity(sizeDto as CreateProductSizeDto)
                    excelDataRow[index].SizeIDs.push(newSizeData);
                } else {
                    if (Object.keys(sizeRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(sizeName.trim()),

                        };
                        let newSizeData = await this.productSizeRepo.createEntity(clrDto as CreateProductSizeDto)
                        excelDataRow[index].SizeIDs.push(newSizeData);
                    } else {
                        excelDataRow[index].SizeIDs.push(sizeRawData[0]);
                    }

                }

            }
            //Size Section End

            //Sale Section Start
            let saleArray = [];
            if (JSON.stringify(rowsData.Sale).length > 0) {
                let saleData = JSON.stringify(rowsData.Sale).trim();
                saleArray = saleData.split(',')
            }

            for (let j = 0; j < saleArray.length; j++) {
                const saleName = saleArray[j];

                let saleRawData = await this.productSaleRepository.findByFilterQuery({
                    name: this.cleanNameString(saleName.trim())
                });

                if (!saleRawData) {
                    let saleDto = {
                        name: this.cleanNameString(saleName.trim()),
                    };
                    let newSaleData = await this.productSaleRepository.createEntity(saleDto as CreateProductSaleDto)
                    excelDataRow[index].SaleIDs.push(newSaleData);
                } else {
                    if (Object.keys(saleRawData).length == 0) {
                        let saleDto = {
                            name: this.cleanNameString(saleName.trim())
                        };
                        let newSaleData = await this.productSaleRepository.createEntity(saleDto as CreateProductSaleDto)
                        excelDataRow[index].SaleIDs.push(newSaleData);
                    } else {
                        excelDataRow[index].SaleIDs.push(saleRawData[0]);
                    }
                }
            }
            // Sale Section End
        }

        excelDataRow.forEach(async row => {
            let dataRow = {
                'DumpID': row.DumpID,
                'Name': row.Name,
                'Sku': row.Sku,
                'ParentCategoryName': row.ParentCategoryName,
                'Cost': row.Cost,
                'Price': row.Price,
                'DiscountPrice': row.DiscountPrice,
                'Stock': row.Stock,
                'Weight': row.Weight,
                'OuterShell': row.OuterShell,
                'Lining': row.Lining,
                'Size': row.Size,
                'Color': row.Color,
                'Sale': row.Sale,
            };
            await this.productDumpImportRepository.createEntity(dataRow as CreateProductDumpImportDto);

        });


        const formattedResponse = {
            dumpID: dumpID,
            dump: excelDataRow
        }


        return ResponseUtils.successResponseHandler(200, "Import Product Data Dump Successfully created", "product_dump", formattedResponse);
    }

    async confirmImportProductService(dto: CreateConfirmImportProductDto): Promise<any> {
        if (dto.dumpID.length > 0) {
            let dumpData = await this.productDumpImportRepository.findByFilterQuery({
                DumpID: dto.dumpID
            });

            if (!dumpData) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }

            for (let index = 0; index < Object.keys(dumpData).length; index++) {
                const row = dumpData[index];

                let crProd = {
                    'name': row.Name,
                    'sku': row.Sku,
                    'parentCategoryId': '',
                    'parentCategoryName': row.ParentCategoryName,
                    'cost': row.Cost,
                    'price': row.Price,
                    'discountPrice': row.DiscountPrice,
                    'stock': row.Stock,
                    'weight': row.Weight,
                    'outerShell': row.OuterShell,
                    'lining': row.Lining,
                    'size': row.Size,
                    'color': row.Color,
                    'sale': row.Sale,
                }


                let parentCategoryData = await this.productCategoryRepository.findByFilterQuery({
                    name: row.ParentCategoryName
                })

                crProd.parentCategoryId = !parentCategoryData || Object.keys(parentCategoryData).length == 0 ? 0 : parentCategoryData[0].id;
                crProd["parentCategoryName"] = row.ParentCategory;

                //Pulling color size temperature wattage weight start 
                //Colors Section Start
                let colorsArray = [];
                if (JSON.stringify(row.Color).length > 0) {
                    let colorsData = JSON.stringify(row.Color).trim();
                    let colorsArrayRaw = colorsData.split(',')


                    for (let index = 0; index < colorsArrayRaw.length; index++) {
                        const element = this.cleanNameString(colorsArrayRaw[index]);
                        colorsArray.push(element);
                    }
                    crProd.color = (colorsArray.length > 0) ? colorsArray : [];
                }
                //Colors Section End


                //Size Section Start
                let sizeArray = [];
                if (JSON.stringify(row.Size).length > 0) {
                    let sizeData = JSON.stringify(row.Size).trim();
                    let sizeArrayRaw = sizeData.split(',')


                    for (let index = 0; index < sizeArrayRaw.length; index++) {
                        const element = this.cleanNameString(sizeArrayRaw[index]);
                        sizeArray.push(element);
                    }
                    crProd.size = (sizeArray.length > 0) ? sizeArray : [];
                }
                //Size Section End

                //Sale Section Start
                let saleArray = [];
                if (JSON.stringify(row.Sale).length > 0) {
                    let saleData = JSON.stringify(row.Sale).trim();
                    let saleArrayRaw = saleData.split(',')


                    for (let index = 0; index < saleArrayRaw.length; index++) {
                        const element = this.cleanNameString(saleArrayRaw[index]);
                        saleArray.push(element);
                    }
                    crProd.sale = (saleArray.length > 0) ? saleArray : [];
                }
                //Sale Section End

                let proCreate = await this.create(crProd as unknown as CreateProductDto, {})
            }

        }

        return ResponseUtils.successResponseHandler(200, "Confirm Import Product Data Dump Successfully Completed", "product_dump_create_product", null);
    }

    async importProductStockService(dto: CreateImportProductDto, file: Express.Multer.File): Promise<any> {
        const stocks = await this.productStockImportRepository.findStockIdEntity();
        const StockID = stocks ? await generateStockId(stocks) : 'hymanfashion';

        const excelDataRow = await readXlsxFile(file.path).then((rows) => {
            let excelDataRowDump = [];


            rows.forEach(async (element, index) => {

                if (index > 0) {
                    let prodId: string = element[0].toString();

                    let productRawData = await {
                        'StockID': StockID,
                        'Pid': prodId,
                        'Name': element[1],
                        'Sku': element[2],
                        'Price': element[3],
                        'CurrentStock': 0,
                        'NewStock': element[4],
                        'NewTotalStock': 0,
                        'QuantityReorder': element[5],
                        'Weight': element[6],
                        'Size': element[7],
                        'SizeIDs': [],
                        'Color': element[8],
                        'ColorIDs': [],
                    };
                    excelDataRowDump.push(productRawData);


                }
            });
            return excelDataRowDump;
        })

        for (let index = 0; index < excelDataRow.length; index++) {
            const rowsData = excelDataRow[index];

            // Colors Section Start
            let colorsArray = [];
            if (JSON.stringify(rowsData.Color).length > 0) {
                let colorsData = JSON.stringify(rowsData.Color).trim();
                colorsArray = colorsData.split(',')

            }

            for (let j = 0; j < colorsArray.length; j++) {
                const colorsName = colorsArray[j];


                let colorsRawData = await this.productColorRepository.findByFilterQuery({
                    colorName: this.cleanNameString(colorsName).trim()
                });

                if (!colorsRawData) {
                    let clrDto = {
                        name: this.cleanNameString(colorsName).trim(),
                        colorCode: '#0000'
                    };
                    let newColorData = await this.productColorRepository.createEntity(clrDto as CreateProductColorDto)
                    excelDataRow[index].ColorIDs.push(newColorData);
                } else {
                    if (Object.keys(colorsRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(colorsName).trim(),
                            colorCode: '#0000'
                        };
                        let newColorData = await this.productColorRepository.createEntity(clrDto as CreateProductColorDto)
                        excelDataRow[index].ColorIDs.push(newColorData);
                    } else {
                        excelDataRow[index].ColorIDs.push(colorsRawData[0]);
                    }

                }
            }
            //Colors Section End


            //Size Section Start
            let sizeArray = [];
            if (JSON.stringify(rowsData.Size).length > 0) {
                let sizeData = JSON.stringify(rowsData.Size).trim();
                sizeArray = sizeData.split(',');

            }

            for (let j = 0; j < sizeArray.length; j++) {
                const sizeName = sizeArray[j];


                let sizeRawData = await this.productSizeRepo.findByFilterQuery({
                    name: this.cleanNameString(sizeName.trim())
                });

                if (!sizeRawData) {
                    let clrDto = {
                        name: this.cleanNameString(sizeName.trim()),
                    };
                    let newSizeData = await this.productSizeRepo.createEntity(clrDto as CreateProductSizeDto)
                    excelDataRow[index].SizeIDs.push(newSizeData);
                } else {
                    if (Object.keys(sizeRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(sizeName.trim())
                        };
                        let newSizeData = await this.productSizeRepo.createEntity(clrDto as CreateProductSizeDto)
                        excelDataRow[index].SizeIDs.push(newSizeData);
                    } else {
                        excelDataRow[index].SizeIDs.push(sizeRawData[0]);
                    }

                }

            }
            //Size Section End
        }



        for (let k = 0; k < excelDataRow.length; k++) {
            const element = excelDataRow[k];
            const id = element.Pid;
            let productInfo = await this.repository.findOne(id);
            if (typeof productInfo === 'undefined') {

            } else {

                let currentQuantity = productInfo["stock"];
                let newQuantity = element.NewStock;
                let totalQuantity = currentQuantity + newQuantity;

                excelDataRow[k]['CurrentStock'] = currentQuantity;
                excelDataRow[k]['NewTotalStock'] = totalQuantity;
            }
        }

        excelDataRow.forEach(async row => {


            let dataRow = {
                'StockID': row.StockID,
                'Pid': row.Pid,
                'Name': row.Name,
                'Price': row.Price,
                'CurrentStock': row.CurrentStock,
                'NewStock': row.NewStock,
                'NewTotalStock': row.NewTotalStock,
                'QuantityReorder': row.QuantityReorder,
                'Sku': row.Sku,
                'Weight': row.Weight,
                'Size': row.Size,
                'Color': row.Color,
            };
            const data = await this.productStockImportRepository.createEntity(dataRow as CreateProductStockImportDto);


        });

        let dataRetrn = {
            stockID: StockID,
            stock: excelDataRow
        }

        return ResponseUtils.successResponseHandler(200, "Import Product Data Stock Successfully created", "product_stock", dataRetrn);
    }


    async confirmStockInProductService(dto: CreateConfirmStockInProductDto): Promise<any> {
        if (dto.stockID.length > 0) {
            let stockData = await this.productStockImportRepository.findByFilterQuery({
                StockID: dto.stockID
            });

            if (!stockData) {
                return ResponseUtils.errorResponseHandler(404, "Stock information not available");
            }

            for (let index = 0; index < Object.keys(stockData).length; index++) {
                const row = stockData[index];


                let crProd = {
                    'name': row.Name,
                    'price': row.Price,
                    'stock': row.NewTotalStock,
                    'weight': row.Weight,
                    'sku': row.Sku,
                    'size': row.Size,
                    'color': row.Color,
                }

                //Colors Section Start
                let colorsArray = [];
                if (JSON.stringify(row.Color).length > 0) {
                    let colorsData = JSON.stringify(row.Color).trim();
                    let colorsArrayRaw = colorsData.split(',');


                    for (let index = 0; index < colorsArrayRaw.length; index++) {
                        const element = this.cleanNameString(colorsArrayRaw[index]);
                        colorsArray.push(element);
                    }
                    crProd.color = (colorsArray.length > 0) ? colorsArray : [];
                }
                //Colors Section End


                //Size Section Start
                let sizeArray = [];
                if (JSON.stringify(row.Size).length > 0) {
                    let sizeData = JSON.stringify(row.Size).trim();
                    let sizeArrayRaw = sizeData.split(',');


                    for (let index = 0; index < sizeArrayRaw.length; index++) {
                        const element = this.cleanNameString(sizeArrayRaw[index]);
                        sizeArray.push(element);
                    }
                    crProd.size = (sizeArray.length > 0) ? sizeArray : [];
                }
                // Size Section End

                let proCreate = await this.updateProductStock(crProd as UpdateProductDto, row.Pid)

            }

        }

        return ResponseUtils.successResponseHandler(200, "Confirm Stockin Product Data Successfully Completed", "product_stockin_product", null);
    }
}
