import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {UpdateSalesTaxCMSDto} from './dto/update-sales-tax-cms.dto';
import {SalesTaxCMS} from './entities/sales-tax-cms.entity';
import {SalesTaxCMSRepository} from './sales-tax-cms.repository';

@Injectable()
export class SalesTaxCMSService {
    constructor(
        @InjectRepository(SalesTaxCMSRepository)
        private readonly repository: SalesTaxCMSRepository,
    ) {
    }

    async updateOrCreateData(
        dto: UpdateSalesTaxCMSDto,
    ): Promise<SalesTaxCMS | Error> {
        const isExists = await this.repository.find();
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.GENERIC_ERROR,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
        }
    }

    async findAll(): Promise<SalesTaxCMS[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
