import { Controller, Get, Param, Query, Req, UseGuards } from '@nestjs/common';
import { Constants } from "src/utils/constants";
import { SiteFrontendService } from "./site-frontend.service";
import { AuthGuard } from 'src/core/guards/auth.guard';
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";

@Controller({
    path: "site",
    version: Constants.API_VERSION_1
})
export class SiteFrontendController {
    constructor(private readonly service: SiteFrontendService) {
    }

    @Get("/common-cms")
    commonCMS() {
        return this.service.commonCMS();
    }

    @Get("/about-us-page")
    aboutUsPage() {
        return this.service.aboutUsPage();
    }

    @Get('/blogs')
    getBlogsData(
        @Query('page') pageParam: string,
        @Query('limit') limitParam: string
    ) {
        const page = parseInt(pageParam, 10) || 1;
        const limit = parseInt(limitParam, 10) || 10;
        return this.service.getAllBlogs(page, limit);
    }


    @Get("/blogs/:slug")
    getBlogDetailsData(
        @Param("slug") slug: string
    ) {
        return this.service.getBlogDetailsData(slug);
    }

    @UseGuards(AuthGuard)
    @Get("/getUserAddresses")
    getUserAddresses(
        @Req() request: any
    ) {
        return this.service.getUserAddresses(request.user.id);
    }

    @UseGuards(AdminAuthGuard)
    @Get("/getUserAddresses/:userId")
    getUserAddressesByAdmin(@Param('userId') userId: string) {
        return this.service.getUserAddresses(userId);
    }

    // Need filtering
    @Get('/products')
    async filterByProductName(
        @Query('limit') limit: any,
        @Query('page') page: any,
        @Query('keyword') keyword?: any,
        @Query('parentCategoryId') parentCategoryId?: any,
        @Query('maxPrice') maxPrice?: any,
        @Query('minPrice') minPrice?: any,
        @Query('color') color?: any,
        @Query('size') size?: any,
        @Query('sale') sale?: any,
        @Query('sortBy') sortBy?: any,
        @Query('readyToShip') readyToShip?: any,
    ) {
        return await this.service.filterByProductName(
            limit,
            page,
            keyword,
            parentCategoryId,
            maxPrice,
            minPrice,
            color,
            size,
            sale,
            sortBy,
            readyToShip,
        );
    }

    @Get('/products/:slug')
    async findOneBySlug(@Param('slug') slug: string) {
        return await this.service.findOneBySlug(slug);
    }
}
