import {Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import axios from 'axios';
import {upsShippingCreate} from 'src/helper/ups-payload';
import {UpsConfigurationRepository} from 'src/modules/ups-configuration/ups-configuration.repository';

@Injectable()
export class UpsShippingService {
    constructor(
        @InjectRepository(UpsConfigurationRepository)
        private readonly repository: UpsConfigurationRepository
    ) {
    }

    async create(dto: any) {
        try {
            const upsConfig = await this.repository.findAll();
            const username = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUserName : upsConfig[0].sandboxUserName;
            const password = upsConfig[0].credentialType == 'live' ? upsConfig[0].livePassword : upsConfig[0].sandboxPassword;
            const accessLicenseNumber = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveAccessLicenseNumber : upsConfig[0].sandboxAccessLicenseNumber;
            const baseUrl = upsConfig[0].credentialType == 'live' ? upsConfig[0].liveUrl : upsConfig[0].sandboxUrl;

            const config = {
                headers: {
                    'Content-Type': "application/json",
                    'AccessLicenseNumber': accessLicenseNumber,
                    'Username': username,
                    'Password': password,
                }
            };

            const service = {
                "02": "2nd Day Air",
                "59": "2nd Day Air A.M.",
                "12": "3 Day Select",
                "03": "Ground UPS",
                "01": "Next Day Air",
                "14": "Next Day Air Early AM",
                "13": "Next Day Air Saver",
            }

            // if (dto.serviceType == "Free Service") {
            //     dto.serviceType = "03";
            // }

            if (dto.weightValue > 150) {
                dto.weightValue = 150;
            }

            const payload = upsShippingCreate(
                dto.shippingAddress.addressOne,
                dto.shippingAddress.city,
                dto.shippingAddress.stateCode,
                dto.shippingAddress.postCode,
                dto.shippingAddress.countryCode || 'US',
                dto.serviceType,
                service[dto.serviceType],
                dto.weightValue,
                dto.shippingAddress.name,
                dto.shippingAddress.mobile,
                upsConfig[0]
            );

            const response = await axios.post(baseUrl + 'ship/v1/shipments', payload, config);
            if (response.status === 200) {
                return response.data
            }
        } catch (e) {
            if (e.response) {
                return {
                    statusCode: e.response.status,
                    data: e.response.data
                }
            } else {
                throw e;
            }
        }
    }
}
