import {VersioningType} from '@nestjs/common';
import {NestFactory} from '@nestjs/core';
import {AppModule} from './app.module';
import {Constants} from './utils/constants';
import * as bodyParser from 'body-parser';
import {DocumentBuilder, SwaggerModule} from '@nestjs/swagger'
import * as fs from 'fs';

async function bootstrap() {
    const httpsOptions = {
      key: fs.readFileSync('./space-secrets/space.key'),
      cert: fs.readFileSync('./space-secrets/space.crt'),
    };
    const app = await NestFactory.create(AppModule, {
      httpsOptions
    });
    // const app = await NestFactory.create(AppModule);
    app.enableCors({
        origin: true,
        methods: 'GET,HEAD,PUT,PATCH,POST,DELETE,OPTIONS',
        credentials: true,
    });
    app.use(bodyParser.json({limit: '50mb'}));
    app.use(bodyParser.urlencoded({limit: '50mb', extended: true}));
    app.setGlobalPrefix(Constants.API);
    app.enableVersioning({
        type: VersioningType.URI,
    });

    const config = new DocumentBuilder()
        .setTitle("Hyman Fashion API List")
        .setDescription("This is Hyman Fashion site API details documentation")
        .setVersion("1.0")
        .build()

    const document = SwaggerModule.createDocument(app, config);
    SwaggerModule.setup('docs', app, document);
    await app.listen(7025);
}

bootstrap();
