import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { UpdateShopPageCmsDto } from './dto/update-shop-page-cms.dto';
import { ShopPageCmsEntity } from './entities/shop-page-cms.entity';
import { ShopPageCmsRepository } from './shop-page-cms.repository';

@Injectable()
export class ShopPageCmsService {
    constructor(
        @InjectRepository(ShopPageCmsRepository)
        private readonly repository: ShopPageCmsRepository,
        private readonly doSpaceService: DoSpacesService,
    ) { }

    async updateOrCreateData(
        dto: UpdateShopPageCmsDto,
        files: {
            topBannerImage: UploadedMulterFileI;
        },
    ): Promise<ShopPageCmsEntity | Error> {
        if (files && files.topBannerImage) {
            const topBannerImage: any = await this.doSpaceService.uploadFile(
                files.topBannerImage[0],
                'ShopPageCms/',
            );
            dto.topBannerImage = topBannerImage;
        }
        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }
            return ResponseUtils.successResponseHandler(
                201,
                'Data created successfully!',
                'data',
                data,
            );
        } else {
            const id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST,
                );
            }
            return ResponseUtils.successResponseHandler(
                200,
                'Data updated successfully!',
                'data',
                data,
            );
        }
    }

    async findAll(): Promise<ShopPageCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            data,
        );
    }
}
