import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateFooterCollectionSectionDto} from "./dto/create-footer-collection-section.dto";
import {UpdateFooterCollectionSectionDto} from "./dto/update-footer-collection-section.dto";
import {FooterCollectionSectionEntity} from "./entity/footer-collection-section.entity";

@EntityRepository(FooterCollectionSectionEntity)
export class FooterCollectionSectionRepository extends Repository<FooterCollectionSectionEntity> {

    async createEntity(dto: CreateFooterCollectionSectionDto): Promise<FooterCollectionSectionEntity | Error> {
        try {
            const {
                collectionName,
                collectionLink
            } = dto;
            const entity = this.create({
                collectionName,
                collectionLink
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<FooterCollectionSectionEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<FooterCollectionSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateFooterCollectionSectionDto): Promise<FooterCollectionSectionEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(FooterCollectionSectionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}