import {EntityRepository, getConnection, Repository} from "typeorm";
import {TrendingSectionEntity} from "./entities/trending-section.entity";
import {CreateTrendingSectionDto} from "./dto/create-trending-section.dto";
import {UpdateTrendingSectionDto} from "./dto/update-trending-section.dto";

@EntityRepository(TrendingSectionEntity)
export class TrendingSectionRepository extends Repository<TrendingSectionEntity> {
    async createEntity(dto: CreateTrendingSectionDto): Promise<TrendingSectionEntity | Error> {
        try {
            const {
                image,
                sectionSubTitle,
            } = dto;
            const entity = this.create({
                image,
                sectionSubTitle
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<TrendingSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<TrendingSectionEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(TrendingSectionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateTrendingSectionDto): Promise<TrendingSectionEntity | Error> {
        try {
            await this.update(id, {
                image: dto.image,
                sectionSubTitle: dto.sectionSubTitle
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }
}