import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductCareVariantsRepository } from './product-care-variants.repository';
import { CreateProductCareVariantsDto } from './dto/create-product-care-variants.dto';
import { ProductCareVariants } from './entities/product-care-variants.entity';
import { UpdateProductCareVariantsDto } from './dto/update-product-care-variants.dto';
@Injectable()
export class ProductCareVariantsService {
    constructor(
        @InjectRepository(ProductCareVariantsRepository)
        private readonly repository: ProductCareVariantsRepository
    ) { }

    async create(dto: CreateProductCareVariantsDto): Promise<ProductCareVariants | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product Color Variants created successfully", "data", data);
    }

    async findAll(): Promise<ProductCareVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Color Variants fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductCareVariantsDto): Promise<ProductCareVariants | Error> {
        const productColorVariant: any = await this.repository.findOneEntity(id);
        if (!productColorVariant) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Color Variants updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductCareVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Color Variants fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product Color Variants deleted successfully");
    }
}
