import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductColorVariantsRepository } from './product-color-variants.repository';
import { CreateProductColorVariantsDto } from './dto/create-product-color-variants.dto';
import { UpdateProductColorVariantsDto } from './dto/update-product-color-variants.dto';
import { ProductColorVariants } from './entities/product-color-variants.entity';

@Injectable()
export class ProductColorVariantsService {
    constructor(
        @InjectRepository(ProductColorVariantsRepository)
        private readonly repository: ProductColorVariantsRepository
    ) { }

    async create(dto: CreateProductColorVariantsDto): Promise<ProductColorVariants | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product Color Variants created successfully", "data", data);
    }

    async findAll(): Promise<ProductColorVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Color Variants fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductColorVariantsDto): Promise<ProductColorVariants | Error> {
        const productColorVariant: any = await this.repository.findOneEntity(id);
        if (!productColorVariant) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Color Variants updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductColorVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Color Variants fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product Color Variants deleted successfully");
    }
}
