import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductOfferVariantsRepository } from './product-offer-variants.repository';
import { ProductOfferVariants } from './entities/product-offer-variants.entity';
import { CreateProductOfferVariantsDto } from './dto/create-product-offer-variants.dto';
import { UpdateProductOfferVariantsDto } from './dto/update-product-offer-variants.dto';

@Injectable()
export class ProductOfferVariantsService {
    constructor(
        @InjectRepository(ProductOfferVariantsRepository)
        private readonly repository: ProductOfferVariantsRepository
    ) { }

    async create(dto: CreateProductOfferVariantsDto): Promise<ProductOfferVariants | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product Sale Variants created successfully", "data", data);
    }

    async findAll(): Promise<ProductOfferVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Sale Variants fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductOfferVariantsDto): Promise<ProductOfferVariants | Error> {
        const category: any = await this.repository.findOneEntity(id);
        if (!category) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Sale Variants updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductOfferVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Sale Variants fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product Sale Variants deleted successfully");
    }
}
