import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { CreateProductSizeDto } from "./dto/create-product-size.dto";
import { UpdateProductSizeDto } from "./dto/update-product-size.dto";
import { ProductSize } from "./entities/product-size.entity";

@EntityRepository(ProductSize)
export class ProductSizeRepository extends Repository<ProductSize> {

    async createEntity(dto: CreateProductSizeDto): Promise<any | Error> {

        try {
            const entity = this.create({
                name: dto.name
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<ProductSize[] | Error> {
        try {
            return await this.find({
                order: {
                    createdAt: -1
                },
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductSize[] | Error> {
        try {
            return await this.find({
                where: {
                    id: id
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntityBySlug(urlSlug: string): Promise<ProductSize[] | Error> {
        try {
            return await this.find({
                where: {
                    urlSlug: urlSlug
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductSize)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async updateProductSize(id: string, dto: UpdateProductSizeDto): Promise<ProductSize | Error> {
        try {
            await this.update(id, {
                name: dto.name
            });
            return await this.findOne({ where: { id: id } });
        } catch (e) {
            return e;
        }
    }

    async getProductSizeList(): Promise<ProductSize[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                },
                take: 3
            });
        } catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductSize[] | Error> {
        try {
            return await this.find({
                where: {
                    name: name,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductSize[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

}