import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateSalesTaxDto} from "./dto/create-sales-tax.dto";
import {UpdateSalesTaxDto} from "./dto/update-sales-tax.dto";
import {SalesTax} from "./entities/sales-tax.entity";

@EntityRepository(SalesTax)
export class SalesTaxRepository extends Repository<SalesTax> {
    async createEntity(dto: CreateSalesTaxDto): Promise<SalesTax | Error> {
        try {
            const {stateName, taxTitle, taxAmount} = dto;
            const entity = this.create({stateName, taxAmount, taxTitle});
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }


    async findAll(): Promise<SalesTax[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<SalesTax | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateSalesTaxDto): Promise<SalesTax | Error> {
        try {
            await this.update(id, {
                stateName: dto.stateName,
                taxTitle: dto.taxTitle,
                taxAmount: dto.taxAmount
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<SalesTax[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(SalesTax)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}