import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from '@nestjs/common';
import {FileFieldsInterceptor} from '@nestjs/platform-express';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {getMulterConfig} from 'src/core/multer.config';
import {Constants} from 'src/utils/constants';
import {CreateShippingMethodCMSDto} from './dto/create-shipping-method-cms-dto.dto';
import {ShippingMethodCMSService} from './shipping-method-cms.service';

@Controller({path: 'shipping-method-cms', version: Constants.API_VERSION_1})

export class ShippingMethodCMSController {
    constructor(private readonly service: ShippingMethodCMSService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            // {name: 'fedexImage', maxCount: 1},
            {name: 'upsImage', maxCount: 1},
            // {name: 'localImage', maxCount: 1}
        ],
    ))
    updateOrCreateData(
        @Body() dto: CreateShippingMethodCMSDto,
        @UploadedFiles() files: {
            // fedexImage?: Express.Multer.File,
            upsImage?: Express.Multer.File,
            // localImage?: Express.Multer.File
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }
}
